// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var Vector2D = require( 'Vector2D' );
  var Resettable = require( 'Resettable' );
  var Property = require( 'Property' );
  var ObservableList = require( 'ObservableList' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var LGConstants = require( 'LGConstants' );
  var Orientation = require( 'Orientation' );

//Class declaration for LineFormsModel

  /**
   * Constructor.
   *
   * @param interactiveLine line that the user can manipulate
   */
  /**
   * Constructor for LineFormsModel
   * @param {Line} interactiveLine
   */
  function LineFormsModel( interactiveLine ) {
    this.graph = new Graph( this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
    var mvtScale = GRID_VIEW_UNITS / Math.max( this.xRange.getLength(), this.yRange.getLength() );
    this.mvt = ModelViewTransform.createOffsetScaleMapping( ORIGIN_OFFSET, mvtScale, -mvtScale );
    this.interactiveLine = new Property( interactiveLine );
    this.savedLines = new ObservableList();
    this.standardLines = new ObservableList();
    var updateGraphLines = new VoidFunction1()
    {

      apply: function( line ) {
        this.lines.clear();
        this.lines.add( this.interactiveLine.get() );
        for ( var l :
        savedLines
      )
        {
          this.lines.add( l );
        }
        for ( var l :
        standardLines
      )
        {
          this.lines.add( l );
        }
      }
    }
    ;
    this.interactiveLine.addObserver( updateGraphLines );
    savedLines.addElementAddedObserver( updateGraphLines );
    savedLines.addElementRemovedObserver( updateGraphLines );
    standardLines.addElementAddedObserver( updateGraphLines );
    standardLines.addElementRemovedObserver( updateGraphLines );
    this.pointTool1 = new PointTool( new Vector2D( this.xRange.getMin() + (0.35 * this.xRange.getLength()), this.yRange.getMin() - 0.25 ), this.UP, this.lines );
    this.pointTool2 = new PointTool( new Vector2D( this.xRange.getMin() + (0.65 * this.xRange.getLength()), this.yRange.getMin() - 2.75 ), this.DOWN, this.lines );
  }

  return inherit( Object, LineFormsModel, {

    reset: function() {
      interactiveLine.reset();
      savedLines.clear();
      standardLines.clear();
      pointTool1.reset();
      pointTool2.reset();
    },


  } );

} );

