// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Vector2D = require( 'Vector2D' );
  var Resettable = require( 'Resettable' );
  var Property = require( 'Property' );
  var ObservableList = require( 'ObservableList' );
  var SimpleObserver = require( 'SimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );

//Class declaration for PointTool

  /**
   * Constructor
   *
   * @param location location of the tool, in model coordinate frame
   * @param lines    lines that the tool might intersect, provided in the order that they would be rendered
   */
  /**
   * Constructor for PointTool
   * @param {Vector2D} location
   * @param {Orientation} orientation
   * @param {ObservableList<Line>} lines
   */
  function PointTool( location, orientation, lines ) {
    this.location = new Property( location );
    this.orientation = orientation;
    this.onLine = new Property( null );
    this.lines = lines;
    {
      this.location.addObserver( new SimpleObserver()
      {

        update: function() {
          this.updateOnLine();
        }
      }
    )
      ;
      var linesChanged = new VoidFunction1()
      {

        apply: function( line ) {
          this.updateOnLine();
        }
      }
      ;
      lines.addElementAddedObserver( linesChanged );
      lines.addElementRemovedObserver( linesChanged );
    }
  }

  return inherit( Object, PointTool, {

    updateOnLine: function() {
      for ( var i = lines.size() - 1; i >= 0; i-- ) {
        var line = lines.get( i );
        if ( this.isOnLine( line ) ) {
          onLine.set( line );
          return;
        }
      }
      onLine.set( null );
    },


    isOnLine: function( line ) {
      if ( this.run == 0 && location.get().getX() == this.x1 ) {
        return true;
      }
      else if ( this.rise == 0 && location.get().getY() == this.y1 ) {
        return true;
      }
      else if ( this.run != 0 && this.rise != 0 && location.get().getY() == line.solveY( location.get().getX() ) ) {
        return true;
      }
      return false;
    },


    reset: function() {
      location.reset();
      onLine.reset();
    },


  } );

} );

