// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Shape = require( 'Shape' );
  var Rectangle2D = require( 'Rectangle2D' );
  var NumberFormat = require( 'NumberFormat' );
  var Property = require( 'Property' );
  var DefaultDecimalFormat = require( 'DefaultDecimalFormat' );
  var DoubleRange = require( 'DoubleRange' );
  var PhetFont = require( 'PhetFont' );
  var PhetPNode = require( 'PhetPNode' );
  var UserComponents = require( 'UserComponents' );
  var RiseSpinnerNode = require( 'RiseSpinnerNode' );
  var RunSpinnerNode = require( 'RunSpinnerNode' );
  var SlopeColors = require( 'SlopeColors' );
  var PNode = require( 'PNode' );
  var PDimension = require( 'PDimension' );

//Class declaration for EquationNode

  /**
   * Constructor.
   *
   * @param pointSize point size of the font used to render the equation.
   */
  /**
   * Constructor for EquationNode
   * @param {int} pointSize
   */
  function EquationNode( pointSize ) {
    slopeSignYOffset = 0;
    slopeSignYFudgeFactor = 0.07 * pointSize;
    operatorYFudgeFactor = 0.07 * pointSize;
    fractionLineYFudgeFactor = 0.07 * pointSize;
    undefinedSlopeYFudgeFactor = 0.07 * pointSize;
    fractionLineThickness = 0.06
    f * pointSize;
    operatorLineSize = new PDimension( 0.54 * pointSize, 0.07 * pointSize );
    signLineSize = new PDimension( 0.54 * pointSize, 0.11 * pointSize );
    integerSignXSpacing = 0.18 * pointSize;
    fractionSignXSpacing = 0.36 * pointSize;
    integerSlopeXSpacing = 0.04 * pointSize;
    fractionalSlopeXSpacing = 0.15 * pointSize;
    operatorXSpacing = 0.25 * pointSize;
    relationalOperatorXSpacing = 0.35 * pointSize;
    parenXSpacing = 0.07 * pointSize;
    ySpacing = 0.1 * pointSize;
    spinnersYSpacing = 0.2 * pointSize;
    slopeYSpacing = 0.4 * pointSize;
  }

  return inherit( PhetPNode, EquationNode, {

    createFractionLineShape: function( length ) {
      return new Rectangle2D.Double( 0, 0, length, fractionLineThickness );
    },


    computeMaxSlopeSpinnerWidth: function( riseRange, runRange, font, format ) {
      var maxRiseNode = new RiseSpinnerNode( this.riseSpinner, new Property( riseRange.get().getMax() ), new Property( runRange.get().getMax() ), riseRange, new SlopeColors(), font, format );
      var minRiseNode = new RiseSpinnerNode( this.riseSpinner, new Property( riseRange.get().getMin() ), new Property( runRange.get().getMax() ), riseRange, new SlopeColors(), font, format );
      var maxRunNode = new RunSpinnerNode( this.riseSpinner, new Property( riseRange.get().getMin() ), new Property( runRange.get().getMax() ), runRange, new SlopeColors(), font, format );
      var minRunNode = new RunSpinnerNode( this.riseSpinner, new Property( riseRange.get().getMin() ), new Property( runRange.get().getMin() ), runRange, new SlopeColors(), font, format );
      var maxRiseWidth = Math.max( maxRiseNode.getFullBoundsReference().getWidth(), minRiseNode.getFullBoundsReference().getWidth() );
      var maxRunWidth = Math.max( maxRunNode.getFullBoundsReference().getWidth(), minRunNode.getFullBoundsReference().getWidth() );
      return Math.max( maxRiseWidth, maxRunWidth );
    },


  } );

} );

