// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var BasicStroke = require( 'BasicStroke' );
  var Color = require( 'Color' );
  var Dimension = require( 'Dimension' );
  var Font = require( 'Font' );
  var Stroke = require( 'Stroke' );
  var Line2D = require( 'Line2D' );
  var Point2D = require( 'Point2D' );
  var Rectangle2D = require( 'Rectangle2D' );
  var Icon = require( 'Icon' );
  var ImageIcon = require( 'ImageIcon' );
  var JFrame = require( 'JFrame' );
  var WindowConstants = require( 'WindowConstants' );
  var Vector2D = require( 'Vector2D' );
  var IntegerRange = require( 'IntegerRange' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var PhetFont = require( 'PhetFont' );
  var PhetPCanvas = require( 'PhetPCanvas' );
  var PhetPNode = require( 'PhetPNode' );
  var DoubleArrowNode = require( 'DoubleArrowNode' );
  var PadBoundsNode = require( 'PadBoundsNode' );
  var PhetPText = require( 'PhetPText' );
  var Strings = require( 'Strings' );
  var Graph = require( 'Graph' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );
  var PText = require( 'PText' );
  var PDimension = require( 'PDimension' );
  var PComposite = require( 'PComposite' );

//Class declaration for GraphNode

  /**
   * Constructor
   *
   * @param graph model element that this node displays
   * @param mvt   transform between model and view coordinate frames
   */
  /**
   * Constructor for GraphNode
   * @param {Graph} graph
   * @param {ModelViewTransform} mvt
   */
  function GraphNode( graph, mvt ) {
    assert( graph.contains( new Vector2D( 0, 0 ) ) && graph.contains( new Vector2D( 1, 1 ) ) );
    this.addChild( new GridNode( graph, mvt ) );
    this.addChild( new XAxisNode( graph, mvt ) );
    this.addChild( new YAxisNode( graph, mvt ) );
  }

  return inherit( PhetPNode, GraphNode, {

    createYEqualsXIcon: function( width, color ) {
      return this.createIcon( width, color, -3, -3, 3, 3 );
    },


    createYEqualsNegativeXIcon: function( width, color ) {
      return this.createIcon( width, color, -3, 3, 3, -3 );
    },


    createIcon: function( width, color, x1, y1, x2, y2 ) {
      var axisRange = new IntegerRange( -3, 3 );
      var graph = new Graph( axisRange, axisRange );
      var mvt = ModelViewTransform.createOffsetScaleMapping( new java.awt.geom.Point2D.Double( 0, 0 ), 15, -15 );
      var graphNode = new GraphNode( graph, mvt );
      var p1 = mvt.modelToView( x1, y1 );
      var p2 = mvt.modelToView( x2, y2 );
      graphNode.addChild( new PPath( new Line2D.Double( p1.getX(), p1.getY(), p2.getX(), p2.getY() ) )
      {

        {
          this.setStrokePaint( color );
          this.setStroke( new BasicStroke( 4
          f
        ))
          ;
        }
      ,

      }
      )
      ;
      graphNode.scale( width / graphNode.getFullBoundsReference().getWidth() );
      return new ImageIcon( new PadBoundsNode( graphNode ).toImage() );
    },


    main: function( args ) {
      var axisRange = new IntegerRange( -10, 10 );
      var graph = new Graph( axisRange, axisRange );
      var mvt = ModelViewTransform.createOffsetScaleMapping( new Point2D.Double( 300, 300 ), 20, -20 );
      var graphNode = new GraphNode( graph, mvt );
      var canvas = new PhetPCanvas();
      canvas.setBackground( this.LIGHT_GRAY );
      canvas.setPreferredSize( new Dimension( 800, 600 ) );
      canvas.getLayer().addChild( graphNode );
      var frame = new JFrame();
      frame.setContentPane( canvas );
      frame.pack();
      frame.setDefaultCloseOperation( this.EXIT_ON_CLOSE );
      frame.setVisible( true );
    },


  } );

} );

