// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Rectangle2D = require( 'Rectangle2D' );
  var Resettable = require( 'Resettable' );
  var CenteredStageCanvas = require( 'CenteredStageCanvas' );
  var ResetAllButtonNode = require( 'ResetAllButtonNode' );
  var LGColors = require( 'LGColors' );
  var LGConstants = require( 'LGConstants' );
  var LineFormsModel = require( 'LineFormsModel' );
  var Orientation = require( 'Orientation' );
  var PNode = require( 'PNode' );

//Class declaration for LineFormsCanvas

  /**
   * Constructor
   *
   * @param model            model container
   * @param viewProperties   properties that are specific to the view, have no model representation
   * @param graphNode        visual representation of the graph
   * @param equationControls controls that affect the equation
   * @param graphControls    controls that affect the graph
   */
  /**
   * Constructor for LineFormsCanvas
   * @param {LineFormsModel} model
   * @param {LineFormsViewProperties} viewProperties
   * @param {PNode} graphNode
   * @param {PNode} equationControls
   * @param {PNode} graphControls
   */
  function LineFormsCanvas( model, viewProperties, graphNode, equationControls, graphControls ) {
    this.setBackground( this.CANVAS );
    this.viewProperties = viewProperties;
    var resetAllButtonNode = new ResetAllButtonNode( new Resettable[]
    { this, model }
  ,
    null, this.CONTROL_FONT_SIZE, this.BLACK, this.RESET_ALL_BUTTON
  )
    {

      {
        this.setConfirmationEnabled( false );
      }
    }
    ;
    this.addChild( equationControls );
    this.addChild( graphControls );
    this.addChild( resetAllButtonNode );
    this.addChild( graphNode );
    {
      var xMargin = 5;
      var ySpacing = 25;
      var availableControlPanelWidth = this.getStageWidth() - graphNode.getFullBoundsReference().getMaxX() - (2 * xMargin);
      if ( equationControls.getFullBoundsReference().getWidth() > availableControlPanelWidth ) {
        equationControls.scale( availableControlPanelWidth / equationControls.getFullBoundsReference().getWidth() );
      }
      if ( graphControls.getFullBoundsReference().getWidth() > availableControlPanelWidth ) {
        graphControls.scale( availableControlPanelWidth / graphControls.getFullBoundsReference().getWidth() );
      }
      var centerX = graphNode.getFullBoundsReference().getMaxX() + xMargin + (availableControlPanelWidth / 2);
      equationControls.setOffset( centerX - (equationControls.getFullBoundsReference().getWidth() / 2), 50 );
      graphControls.setOffset( centerX - (graphControls.getFullBoundsReference().getWidth() / 2), equationControls.getFullBoundsReference().getMaxY() + ySpacing );
      resetAllButtonNode.setOffset( centerX - (resetAllButtonNode.getFullBoundsReference().getWidth() / 2), graphControls.getFullBoundsReference().getMaxY() + ySpacing );
    }
    {
      var height = new PointToolNode( this.pointTool1, this.mvt, this.graph, this.getStageBounds(), this.linesVisible ).getFullBoundsReference().getHeight();
      var stageBounds = this.getStageBounds();
      var upDragBounds = new Rectangle2D.Double( stageBounds.getX(), stageBounds.getY() - (height / 2), stageBounds.getWidth(), stageBounds.getHeight() );
      var downDragBounds = new Rectangle2D.Double( upDragBounds.getX(), stageBounds.getY() + (height / 2), upDragBounds.getWidth(), upDragBounds.getHeight() );
      var pointTool1 = new PointToolNode( this.pointTool1, this.mvt, this.graph, this.orientation == this.UP ? upDragBounds : downDragBounds, this.linesVisible );
      var pointTool2 = new PointToolNode( this.pointTool2, this.mvt, this.graph, this.orientation == this.UP ? upDragBounds : downDragBounds, this.linesVisible );
      var pointToolParent = new PNode();
      pointToolParent.addChild( pointTool1 );
      pointToolParent.addChild( pointTool2 );
      this.addChild( pointToolParent );
    }
  }

  return inherit( CenteredStageCanvas, LineFormsCanvas, {

    reset: function() {
      super.reset();
      viewProperties.reset();
    },


  } );

} );

