// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var FunctionHighlightHandler = require( 'FunctionHighlightHandler' );
  var LGColors = require( 'LGColors' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var LineFormsModel = require( 'LineFormsModel' );
  var PNode = require( 'PNode' );
  var PComposite = require( 'PComposite' );

//Class declaration for LineFormsGraphNode

  /**
   * Constructor
   *
   * @param model the model
   * @param viewProperties properties that are specific to the view
   */
  /**
   * Constructor for LineFormsGraphNode
   * @param {LineFormsModel} model
   * @param {LineFormsViewProperties} viewProperties
   */
  function LineFormsGraphNode( model, viewProperties ) {
    super( this.graph, this.mvt );
    this.model = model;
    this.viewProperties = viewProperties;
    standardLinesParentNode = new PComposite();
    savedLinesParentNode = new PNode();
    interactiveLineParentNode = new PComposite();
    slopeToolNode = new SlopeToolNode( this.interactiveLine, this.mvt );
    this.addChild( interactiveLineParentNode );
    this.addChild( savedLinesParentNode );
    this.addChild( standardLinesParentNode );
    this.addChild( slopeToolNode );
    this.standardLines.addElementAddedObserver( new VoidFunction1()
    {

      apply: function( line ) {
        this.standardLineAdded( line );
      }
    }
  )
    ;
    this.standardLines.addElementRemovedObserver( new VoidFunction1()
    {

      apply: function( line ) {
        this.standardLineRemoved( line );
      }
    }
  )
    ;
    this.savedLines.addElementAddedObserver( new VoidFunction1()
    {

      apply: function( line ) {
        this.savedLineAdded( line );
      }
    }
  )
    ;
    this.savedLines.addElementRemovedObserver( new VoidFunction1()
    {

      apply: function( line ) {
        this.savedLineRemoved( line );
      }
    }
  )
    ;
    this.interactiveLine.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        this.updateInteractiveLine( line );
      }
    }
  )
    ;
    var visibilityObserver = new RichSimpleObserver()
    {

      update: function() {
        this.updateLinesVisibility( this.linesVisible.get(), this.interactiveLineVisible.get(), this.slopeVisible.get() );
      }
    }
    ;
    visibilityObserver.observe( this.linesVisible, this.interactiveLineVisible, this.slopeVisible );
    this.interactiveEquationVisible.addObserver( new VoidFunction1()
    {

      apply: function( visible ) {
        if ( interactiveLineNode != null ) {
          interactiveLineNode.setEquationVisible( visible );
        }
      }
    }
  )
    ;
  }

  return inherit( GraphNode, LineFormsGraphNode, {

    getManipulatorDiameter: function() {
      return this.mvt.modelToViewDeltaX( MANIPULATOR_DIAMETER );
    },


    standardLineAdded: function( line ) {
      standardLinesParentNode.addChild( this.createLineNode( line, this.graph, this.mvt ) );
    },


    standardLineRemoved: function( line ) {
      this.removeLineNode( line, standardLinesParentNode );
    },


    savedLineAdded: function( line ) {
      var lineNode = this.createLineNode( line, this.graph, this.mvt );
      savedLinesParentNode.addChild( lineNode );
      lineNode.addInputEventListener( new FunctionHighlightHandler( new VoidFunction1()
      {

        apply: function( highlighted ) {
          lineNode.updateColor( highlighted ? this.SAVED_LINE_HIGHLIGHT : this.color );
        }
      ,

      }
      ))
      ;
    },


    savedLineRemoved: function( line ) {
      this.removeLineNode( line, savedLinesParentNode );
    },


    removeLineNode: function( line, parent ) {
      for ( var i = 0; i < parent.getChildrenCount(); i++ ) {
        var node = parent.getChild( i );
        if ( node instanceof LineNode ) {
          var lineNode = node;
          if ( this.line == line ) {
            parent.removeChild( node );
            break;
          }
        }
      }
    },


    updateLinesVisibility: function( linesVisible, interactiveLineVisible, slopeVisible ) {
      if ( interactiveLineParentNode != null ) {
        interactiveLineParentNode.setVisible( linesVisible && interactiveLineVisible );
      }
      savedLinesParentNode.setVisible( linesVisible );
      standardLinesParentNode.setVisible( linesVisible );
      slopeToolNode.setVisible( slopeVisible && linesVisible && interactiveLineVisible );
    },


    updateInteractiveLine: function( line ) {
      interactiveLineParentNode.removeAllChildren();
      interactiveLineNode = this.createLineNode( line, this.graph, this.mvt );
      interactiveLineNode.setEquationVisible( this.interactiveEquationVisible.get() );
      interactiveLineParentNode.addChild( interactiveLineNode );
    },


    createLineNode: function( line, graph, mvt );
  ,


} );

} )
;

