// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var RoundRectangle2D = require( 'RoundRectangle2D' );
  var NumberFormat = require( 'NumberFormat' );
  var PhetFont = require( 'PhetFont' );
  var PhetPText = require( 'PhetPText' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );
  var PComposite = require( 'PComposite' );

//Class declaration for NumberBackgroundNode

  /**
   * Constructor for NumberBackgroundNode
   * @param {double} value
   * @param {NumberFormat} format
   * @param {PhetFont} font
   * @param {Color} textColor
   * @param {Color} backgroundColor
   * @param {double} xMargin
   * @param {double} yMargin
   * @param {double} cornerRadius
   */
  function NumberBackgroundNode( value, format, font, textColor, backgroundColor, xMargin, yMargin, cornerRadius ) {
    this( value, format, font, textColor, backgroundColor, xMargin, yMargin, cornerRadius, 0 );
  }

  /**
   * Use this constructor if you want to specify a minimum width for the value.
   *
   * @param value the value to display
   * @param format number format of the value
   * @param font font used to render the value
   * @param textColor color of the value
   * @param backgroundColor background color
   * @param xMargin margin on left and right
   * @param yMargin margin on top and bottom
   * @param cornerRadius radius of the background's corners
   * @param minWidth        the minimum width of the value (xMargin is added to this to determine background width)
   */
  /**
   * Constructor for NumberBackgroundNode
   * @param {double} value
   * @param {NumberFormat} format
   * @param {PhetFont} font
   * @param {Color} textColor
   * @param {Color} backgroundColor
   * @param {double} xMargin
   * @param {double} yMargin
   * @param {double} cornerRadius
   * @param {double} minWidth
   */
  function NumberBackgroundNode( value, format, font, textColor, backgroundColor, xMargin, yMargin, cornerRadius, minWidth ) {
    var textNode = new PhetPText( format.format( value ), font, textColor );
    var textWidth = textNode.getFullBoundsReference().getWidth();
    var backgroundWidth = (minWidth == 0) ? textWidth : Math.max( minWidth, textWidth );
    var backgroundShape = new RoundRectangle2D.Double( 0, 0, backgroundWidth + (2 * xMargin), textNode.getFullBoundsReference().getHeight() + (2 * yMargin), cornerRadius, cornerRadius );
    var backgroundNode = new PPath( backgroundShape );
    backgroundNode.setPaint( backgroundColor );
    backgroundNode.setStroke( null );
    this.addChild( backgroundNode );
    this.addChild( textNode );
    textNode.setOffset( backgroundNode.getFullBoundsReference().getCenterX() - (textNode.getFullBoundsReference().getWidth() / 2), yMargin );
  }

  return inherit( PComposite, NumberBackgroundNode, {

  } );

} );

