// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Dimension = require( 'Dimension' );
  var Paint = require( 'Paint' );
  var JFrame = require( 'JFrame' );
  var WindowConstants = require( 'WindowConstants' );
  var DoubleGeneralPath = require( 'DoubleGeneralPath' );
  var PhetPCanvas = require( 'PhetPCanvas' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );
  var PDimension = require( 'PDimension' );

//Class declaration for PlusNode

  /**
   * Constructor for PlusNode
   * @param {PDimension} size
   * @param {Paint} paint
   */
  function PlusNode( size, paint ) {
    this( size.getWidth(), size.getHeight(), paint );
  }

  /**
   * Constructor
   *
   * @param width  width of the horizontal bar
   * @param height height of the horizontal bar
   * @param paint  paint used to fill the plus sign
   */
  /**
   * Constructor for PlusNode
   * @param {double} width
   * @param {double} height
   * @param {Paint} paint
   */
  function PlusNode( width, height, paint ) {
    assert( width > height );
    var c1 = (width / 2) - (height / 2);
    var c2 = (width / 2) + (height / 2);
    var path = new DoubleGeneralPath();
    path.moveTo( c1, 0 );
    path.lineTo( c2, 0 );
    path.lineTo( c2, c1 );
    path.lineTo( width, c1 );
    path.lineTo( width, c2 );
    path.lineTo( c2, c2 );
    path.lineTo( c2, width );
    path.lineTo( c1, width );
    path.lineTo( c1, c2 );
    path.lineTo( 0, c2 );
    path.lineTo( 0, c1 );
    path.lineTo( c1, c1 );
    path.closePath();
    this.setPathTo( path.getGeneralPath() );
    this.setStroke( null );
    this.setPaint( paint );
  }

  return inherit( PPath, PlusNode, {

    main: function( args ) {
      var plusNode = new PlusNode( 60, 5, this.BLACK );
      plusNode.setOffset( 100, 100 );
      var canvas = new PhetPCanvas();
      canvas.setPreferredSize( new Dimension( 800, 600 ) );
      canvas.getLayer().addChild( plusNode );
      var frame = new JFrame();
      frame.setContentPane( canvas );
      frame.pack();
      frame.setDefaultCloseOperation( this.EXIT_ON_CLOSE );
      frame.setVisible( true );
    },


  } );

} );

