// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var IUserComponentType = require( 'IUserComponentType' );
  var ParameterSet = require( 'ParameterSet' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var SimSharingDragHandler = require( 'SimSharingDragHandler' );
  var ParameterKeys = require( 'ParameterKeys' );
  var Line = require( 'Line' );
  var PInputEvent = require( 'PInputEvent' );

//Class declaration for LineManipulatorDragHandler

  /**
   * Constructor
   *
   * @param userComponent   sim-sharing component identifier
   * @param componentType   sim-sharing component type
   * @param manipulatorNode the node being manipulated by the user
   * @param mvt             transform between model and view coordinate frames
   * @param line            the line being manipulated
   */
  /**
   * Constructor for LineManipulatorDragHandler
   * @param {IUserComponent} userComponent
   * @param {IUserComponentType} componentType
   * @param {LineManipulatorNode} manipulatorNode
   * @param {ModelViewTransform} mvt
   * @param {Property<Line>} line
   */
  function LineManipulatorDragHandler( userComponent, componentType, manipulatorNode, mvt, line ) {
    super( userComponent, componentType, true );
    this.manipulatorNode = manipulatorNode;
    this.mvt = mvt;
    this.line = line;
  }

  return inherit( SimSharingDragHandler, LineManipulatorDragHandler, {

    getParametersForAllEvents: function( event ) {
      return new ParameterSet().with( this.x1, this.x1 ).with( this.y1, this.y1 ).with( this.x2, this.x2 ).with( this.y2, this.y2 ).with( this.rise, this.rise ).with( this.run, this.run ).with( super.getParametersForAllEvents( event ) );
    },


  } );

} );

