// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var MathUtil = require( 'MathUtil' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var IUserComponentType = require( 'IUserComponentType' );
  var DoubleRange = require( 'DoubleRange' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var Line = require( 'Line' );
  var PInputEvent = require( 'PInputEvent' );

//Class declaration for SlopeDragHandler

  /**
   * Constructor
   *
   * @param userComponent   sim-sharing component identifier
   * @param componentType   sim-sharing component type
   * @param manipulatorNode the node being manipulated by the user
   * @param mvt             transform between model and view coordinate frames
   * @param line            the line being manipulated
   * @param riseRange       range of the rise
   * @param runRange        range of the run
   */
  /**
   * Constructor for SlopeDragHandler
   * @param {IUserComponent} userComponent
   * @param {IUserComponentType} componentType
   * @param {LineManipulatorNode} manipulatorNode
   * @param {ModelViewTransform} mvt
   * @param {Property<Line>} line
   * @param {Property<DoubleRange>} riseRange
   * @param {Property<DoubleRange>} runRange
   */
  function SlopeDragHandler( userComponent, componentType, manipulatorNode, mvt, line, riseRange, runRange ) {
    super( userComponent, componentType, manipulatorNode, mvt, line );
    this.riseRange = riseRange;
    this.runRange = runRange;
  }

  return inherit( LineManipulatorDragHandler, SlopeDragHandler, {

    startDrag: function( event ) {
      super.startDrag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      clickXOffset = pMouse.getX() - mvt.modelToViewDeltaX( this.x2 );
      clickYOffset = pMouse.getY() - mvt.modelToViewDeltaY( this.y2 );
    },


    drag: function( event ) {
      super.drag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      var run = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaX( pMouse.getX() - clickXOffset ) - this.x1, runRange.get() ) );
      var rise = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaY( pMouse.getY() - clickYOffset ) - this.y1, riseRange.get() ) );
      if ( !(run == 0 && rise == 0) ) {
        line.set( Line.createPointSlope( this.x1, this.y1, rise, run, this.color ) );
      }
    },


  } );

} );

