// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var MathUtil = require( 'MathUtil' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var IUserComponentType = require( 'IUserComponentType' );
  var DoubleRange = require( 'DoubleRange' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var Line = require( 'Line' );
  var PInputEvent = require( 'PInputEvent' );

//Class declaration for X2Y2DragHandler

  /**
   * Constructor
   *
   * @param userComponent   sim-sharing component identifier
   * @param componentType   sim-sharing component type
   * @param manipulatorNode the node being manipulated by the user
   * @param mvt             transform between model and view coordinate frames
   * @param line            the line being manipulated
   * @param x2Range         range of the x coordinate
   * @param y2Range         range of the y coordinate
   */
  /**
   * Constructor for X2Y2DragHandler
   * @param {IUserComponent} userComponent
   * @param {IUserComponentType} componentType
   * @param {LineManipulatorNode} manipulatorNode
   * @param {ModelViewTransform} mvt
   * @param {Property<Line>} line
   * @param {Property<DoubleRange>} x2Range
   * @param {Property<DoubleRange>} y2Range
   */
  function X2Y2DragHandler( userComponent, componentType, manipulatorNode, mvt, line, x2Range, y2Range ) {
    super( userComponent, componentType, manipulatorNode, mvt, line );
    this.x2Range = x2Range;
    this.y2Range = y2Range;
  }

  return inherit( LineManipulatorDragHandler, X2Y2DragHandler, {

    startDrag: function( event ) {
      super.startDrag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      clickXOffset = pMouse.getX() - mvt.modelToViewDeltaX( this.x2 );
      clickYOffset = pMouse.getY() - mvt.modelToViewDeltaY( this.y2 );
    },


    drag: function( event ) {
      super.drag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      var x2 = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaX( pMouse.getX() - clickXOffset ), x2Range.get() ) );
      var y2 = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaY( pMouse.getY() - clickYOffset ), y2Range.get() ) );
      if ( !(x2 == this.x1 && y2 == this.y1) ) {
        line.set( new Line( this.x1, this.y1, x2, y2, this.color ) );
      }
    },


  } );

} );

