// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var MathUtil = require( 'MathUtil' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var IUserComponentType = require( 'IUserComponentType' );
  var DoubleRange = require( 'DoubleRange' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var Line = require( 'Line' );
  var PInputEvent = require( 'PInputEvent' );

//Class declaration for YInterceptDragHandler

  /**
   * Constructor
   *
   * @param userComponent   sim-sharing component identifier
   * @param componentType   sim-sharing component type
   * @param manipulatorNode the node being manipulated by the user
   * @param mvt             transform between model and view coordinate frames
   * @param line            the line being manipulated
   * @param y1Range         range of the y-intercept
   */
  /**
   * Constructor for YInterceptDragHandler
   * @param {IUserComponent} userComponent
   * @param {IUserComponentType} componentType
   * @param {LineManipulatorNode} manipulatorNode
   * @param {ModelViewTransform} mvt
   * @param {Property<Line>} line
   * @param {Property<DoubleRange>} y1Range
   */
  function YInterceptDragHandler( userComponent, componentType, manipulatorNode, mvt, line, y1Range ) {
    super( userComponent, componentType, manipulatorNode, mvt, line );
    this.y1Range = y1Range;
  }

  return inherit( LineManipulatorDragHandler, YInterceptDragHandler, {

    startDrag: function( event ) {
      super.startDrag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      clickYOffset = pMouse.getY() - mvt.modelToViewDeltaY( this.y1 );
    },


    drag: function( event ) {
      super.drag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      var y1 = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaY( pMouse.getY() - clickYOffset ), y1Range.get() ) );
      line.set( Line.createSlopeIntercept( this.rise, this.run, y1, this.color ) );
    },


  } );

} );

