// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var NumberFormat = require( 'NumberFormat' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var DoubleRange = require( 'DoubleRange' );
  var Function0 = require( 'Function0' );
  var PhetFont = require( 'PhetFont' );

//Class declaration for CoordinateSpinnerNode

  /**
   * Constructor
   * @param userComponent used in sim-sharing message
   * @param x1 the coordinate that this spinner changes (not necessarily on the x-axis!)
   * @param y1 the other coordinate of the point that has coordinate x1
   * @param x2 the coordinate in the second point that is on the same axis as x1
   * @param y2 the coordinate in the second point that is on the same axis as y1
   * @param range range of x1
   * @param colors color scheme that will be used for the spinner
   * @param font font for the number in the spinner
   * @param format format of the number in the spinner
   */
  /**
   * Constructor for CoordinateSpinnerNode
   * @param {IUserComponent} userComponent
   * @param {Property<Double>} x1
   * @param {Property<Double>} y1
   * @param {Property<Double>} x2
   * @param {Property<Double>} y2
   * @param {Property<DoubleRange>} range
   * @param {SpinnerStateIndicator<Color>} colors
   * @param {PhetFont} font
   * @param {NumberFormat} format
   */
  function CoordinateSpinnerNode( userComponent, x1, y1, x2, y2, range, colors, font, format ) {
    super( userComponent, x1, range, colors, font, format, new Function0()
    {

      apply: function() {
        var x1New = x1.get() + 1;
        if ( x1New == x2.get().doubleValue() && y1.get().doubleValue() == y2.get().doubleValue() ) {
          x1New++;
          if ( x1New > range.get().getMax() ) {
            x1New = x1.get();
          }
        }
        return x1New;
      }
    }
  ,
    new Function0()
    {

      apply: function() {
        var x1New = x1.get() - 1;
        if ( x1New == x2.get().doubleValue() && y1.get().doubleValue() == y2.get().doubleValue() ) {
          x1New--;
          if ( x1New < range.get().getMin() ) {
            x1New = x1.get();
          }
        }
        return x1New;
      }
    }
  )
    ;
  }

  return inherit( SpinnerNode, CoordinateSpinnerNode, {

  } );

} );

