// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var NumberFormat = require( 'NumberFormat' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var DoubleRange = require( 'DoubleRange' );
  var Function0 = require( 'Function0' );
  var PhetFont = require( 'PhetFont' );

//Class declaration for SlopeSpinnerNode

  /**
   * Constructor.
   *
   * @param userComponent     user component in data-collection messages
   * @param variableComponent the part of the slope that this spinner is manipulating
   * @param fixedComponent    the part of the slope that this spinner is not manipulating
   * @param variableRange     the variableRange of variableComponent
   * @param colors            color scheme for the various states of the spinner
   * @param font              font used to render the value
   * @param format            number format of the value
   */
  /**
   * Constructor for SlopeSpinnerNode
   * @param {IUserComponent} userComponent
   * @param {Property<Double>} variableComponent
   * @param {Property<Double>} fixedComponent
   * @param {Property<DoubleRange>} variableRange
   * @param {SpinnerStateIndicator<Color>} colors
   * @param {PhetFont} font
   * @param {NumberFormat} format
   */
  function SlopeSpinnerNode( userComponent, variableComponent, fixedComponent, variableRange, colors, font, format ) {
    super( userComponent, variableComponent, variableRange, colors, font, format, new Function0()
    {

      apply: function() {
        return (variableComponent.get() == -1 && fixedComponent.get() == 0) ? 1d
      :
        variableComponent.get() + 1;
      }
    }
  ,
    new Function0()
    {

      apply: function() {
        return (variableComponent.get() == 1 && fixedComponent.get() == 0) ? -1d
      :
        variableComponent.get() - 1;
      }
    }
  )
    ;
  }

  return inherit( SpinnerNode, SlopeSpinnerNode, {

  } );

} );

