// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var ArrayList = require( 'ArrayList' );
  var List = require( 'List' );
  var Random = require( 'Random' );
  var IntegerRange = require( 'IntegerRange' );
  var Fraction = require( 'Fraction' );

//Class declaration for ChallengeFactory

  /**
   * Constructor for ChallengeFactory

   */
  function ChallengeFactory() {
    random = new Random();
    fractionChooser = new RandomChooser( random );
    integerChooser = new RandomChooser( random );
    pointChooser = new RandomChooser( random );
    manipulationModeChooser = new RandomChooser( random );
    equationFormChooser = new RandomChooser( random );
  }

  return inherit( Object, ChallengeFactory, {

    /**
     * Creates challenges for the factory's game level.
     *
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     * @return list of challenges
     */
    createChallenges: function( xRange, yRange );
  ,


  rangeToList: function( range ) {
    var list = new ArrayList();
    for ( var i = range.getMin(); i <= range.getMax(); i++ ) {
      list.add( i );
    }
    assert( list.size() > 0 );
    return list;
  }
  ,


  choosePointForSlope: function( slope, graphXRange, graphYRange ) {
    var rise = this.numerator;
    var run = this.denominator;
    var xRange;
    if ( run >= 0 ) {
      xRange = new IntegerRange( graphXRange.getMin(), graphYRange.getMax() - run );
    }
    else {
      xRange = new IntegerRange( graphXRange.getMin() - run, graphYRange.getMax() );
    }
    var xList = this.rangeToList( xRange );
    var yRange;
    if ( rise >= 0 ) {
      yRange = new IntegerRange( graphYRange.getMin(), graphYRange.getMax() - rise );
    }
    else {
      yRange = new IntegerRange( graphYRange.getMin() - rise, graphYRange.getMax() );
    }
    var yList = this.rangeToList( yRange );
    var x = xList.get( this.randomIndex( xList ) );
    var y = yList.get( this.randomIndex( yList ) );
    return new Point2D.Double( x, y );
  }
  ,


  choosePointForSlopeInversion: function( slope, graphXRange, graphYRange ) {
    var rise = this.numerator;
    var run = this.denominator;
    var xRange;
    if ( run >= 0 ) {
      xRange = new IntegerRange( graphXRange.getMax() - run + 1, graphXRange.getMax() );
    }
    else {
      xRange = new IntegerRange( graphXRange.getMin(), graphXRange.getMin() - run - 1 );
    }
    var xList = this.rangeToList( xRange );
    var yRange;
    if ( rise >= 0 ) {
      yRange = new IntegerRange( graphYRange.getMax() - rise + 1, graphYRange.getMax() );
    }
    else {
      yRange = new IntegerRange( graphYRange.getMin(), graphYRange.getMin() - rise - 1 );
    }
    var yList = this.rangeToList( yRange );
    var x1 = xList.get( this.randomIndex( xList ) );
    var y1 = yList.get( this.randomIndex( yList ) );
    var x2 = x1 + run;
    var y2 = y1 + rise;
    assert( graphXRange.contains( x1 ) && !graphXRange.contains( x2 ) );
    assert( graphYRange.contains( y1 ) && !graphYRange.contains( y2 ) );
    return new Point2D.Double( x1, y1 );
  }
  ,


  randomIndex: function( list ) {
    return random.nextInt( list.size() );
  }
  ,


} );

} )
;

