// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var ArrayList = require( 'ArrayList' );
  var Collections = require( 'Collections' );
  var IntegerRange = require( 'IntegerRange' );
  var Fraction = require( 'Fraction' );
  var Line = require( 'Line' );

//Class declaration for ChallengeFactory1
  return inherit( ChallengeFactory, ChallengeFactory1, {

    /**
     * Creates challenges for this game level.
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     * @return list of challenges
     */
    createChallenges: function( xRange, yRange ) {
      var challenges = new ArrayList();
      var x1Range = new IntegerRange( -9, 4 );
      var y1Range = new IntegerRange( -9, 4 );
      var pointLists = new ArrayList()
      {

        {
          this.add( new ArrayList()
          {

            {
              assert( x1Range.getMax() > 0 && y1Range.getMax() > 0 );
              for ( var x = 1; x < x1Range.getMax(); x++ ) {
                for ( var y = 1; y < y1Range.getMax(); y++ ) {
                  this.add( new Point2D.Double( x, y ) );
                }
              }
            }
          ,

          }
        )
          ;
          this.add( new ArrayList()
          {

            {
              assert( x1Range.getMin() < 0 && y1Range.getMin() < 0 );
              for ( var x = x1Range.getMin(); x < 0; x++ ) {
                for ( var y = y1Range.getMin(); y < 0; y++ ) {
                  this.add( new Point2D.Double( x, y ) );
                }
              }
            }
          ,

          }
        )
          ;
        }
      ,

      }
      ;
      var pointListIndices = this.rangeToList( new IntegerRange( 0, pointLists.size() - 1 ) );
      var slopeLists = new ArrayList()
      {

        {
          this.add( new ArrayList()
          {

            {
              this.add( new Fraction( 3, 2 ) );
              this.add( new Fraction( 4, 3 ) );
              this.add( new Fraction( 5, 2 ) );
              this.add( new Fraction( 5, 3 ) );
            }
          ,

          }
        )
          ;
          this.add( new ArrayList()
          {

            {
              this.add( new Fraction( 1, 2 ) );
              this.add( new Fraction( 1, 3 ) );
              this.add( new Fraction( 1, 4 ) );
              this.add( new Fraction( 1, 5 ) );
            }
          ,

          }
        )
          ;
          this.add( new ArrayList()
          {

            {
              this.add( new Fraction( 2, 3 ) );
              this.add( new Fraction( 3, 4 ) );
              this.add( new Fraction( 3, 5 ) );
              this.add( new Fraction( 2, 5 ) );
            }
          ,

          }
        )
          ;
        }
      ,

      }
      ;
      var slopeListIndices = this.rangeToList( new IntegerRange( 0, slopeLists.size() - 1 ) );
      var yInterceptRange = new IntegerRange( -6, 4 );
      var yInterceptLists = new ArrayList()
      {

        {
          assert( yInterceptRange.getMin() < 0 && yInterceptRange.getMax() > 0 );
          this.add( this.rangeToList( new IntegerRange( yInterceptRange.getMin(), -1 ) ) );
          this.add( this.rangeToList( new IntegerRange( 1, yInterceptRange.getMax() ) ) );
        }
      ,

      }
      ;
      var yInterceptListIndices = this.rangeToList( new IntegerRange( 0, yInterceptLists.size() - 1 ) );
      var pointSlopeManipulationModes = new ArrayList()
      {

        {
          this.add( this.POINT );
          this.add( this.SLOPE );
        }
      ,

      }
      ;
      {
        var slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists );
        challenges.add( new GraphTheLine( "1 of 3 required slopes", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists, yInterceptListIndices );
        challenges.add( new GraphTheLine( "1 of 2 required y-intercepts", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.INTERCEPT, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists );
        challenges.add( new MakeTheEquation( "2 of 3 required slopes", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists, yInterceptListIndices );
        challenges.add( new MakeTheEquation( "2 of 2 required y-intercepts", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.INTERCEPT, xRange, yRange ) );
      }
      {
        var manipulationMode = manipulationModeChooser.choose( pointSlopeManipulationModes );
        var point;
        var slope;
        var description;
        if ( manipulationMode == this.SLOPE ) {
          point = pointChooser.chooseFromLists( pointLists );
          slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
          description = "random choice to manipulate slope, 3 of 3 required slopes";
        }
        else {
          point = pointChooser.chooseFromLists( pointLists, pointListIndices );
          slope = fractionChooser.chooseFromLists( slopeLists );
          description = "random choice to manipulate point, 1 of 2 required points";
        }
        challenges.add( new GraphTheLine( description, Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, manipulationMode, xRange, yRange ) );
      }
      {
        var manipulationMode = manipulationModeChooser.choose( pointSlopeManipulationModes );
        var point;
        var slope;
        var description;
        if ( manipulationMode == this.SLOPE ) {
          point = pointChooser.chooseFromLists( pointLists );
          slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
          description = "manipulate slope because Graph-the-Line uses point, 3 of 3 required slopes";
        }
        else {
          point = pointChooser.chooseFromLists( pointLists, pointListIndices );
          slope = fractionChooser.chooseFromLists( slopeLists );
          description = "manipulate point because Graph-the-Line uses slope, 2 of 2 required points";
        }
        challenges.add( new MakeTheEquation( description, Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, manipulationMode, xRange, yRange ) );
      }
      Collections.shuffle( challenges );
      return challenges;
    },


  } );

} );

