// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var ArrayList = require( 'ArrayList' );
  var Collections = require( 'Collections' );
  var IntegerRange = require( 'IntegerRange' );
  var Fraction = require( 'Fraction' );
  var Line = require( 'Line' );

//Class declaration for ChallengeFactory4
  return inherit( ChallengeFactory, ChallengeFactory4, {

    /**
     * Creates challenges for this game level.
     *
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     * @return list of challenges
     */
    createChallenges: function( xRange, yRange ) {
      var challenges = new ArrayList();
      var slopeLists = ChallengeFactory2.createSlopeLists();
      var slopeBinIndices = this.rangeToList( new IntegerRange( 0, slopeLists.size() - 1 ) );
      var yInterceptRange = new IntegerRange( -10, 10 );
      var yInterceptLists = new ArrayList()
      {

        {
          assert( yInterceptRange.getMin() < 0 && yInterceptRange.getMax() > 0 );
          this.add( this.rangeToList( new IntegerRange( yInterceptRange.getMin(), -1 ) ) );
          this.add( this.rangeToList( new IntegerRange( 1, yInterceptRange.getMax() ) ) );
        }
      ,

      }
      ;
      var yInterceptListIndices = this.rangeToList( new IntegerRange( 0, yInterceptLists.size() - 1 ) );
      var equationForms = new ArrayList()
      {

        {
          this.add( this.SLOPE_INTERCEPT );
          this.add( this.POINT_SLOPE );
        }
      ,

      }
      ;
      {
        var slope = fractionChooser.chooseFromLists( slopeLists );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists, yInterceptListIndices );
        challenges.add( new MakeTheEquation( "1 of 2 required y-intercepts", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists, slopeBinIndices );
        var point = this.choosePointForSlope( slope, xRange, yRange );
        challenges.add( new MakeTheEquation( "1 of 3 required slopes", Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
      }
      {
        if ( equationFormChooser.choose( equationForms ) == this.SLOPE_INTERCEPT ) {
          var slope = fractionChooser.chooseFromLists( slopeLists );
          var yIntercept = integerChooser.chooseFromLists( yInterceptLists );
          challenges.add( new MakeTheEquation( "random choice of slope-intercept", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
        }
        else {
          var slope = fractionChooser.chooseFromLists( slopeLists, slopeBinIndices );
          var point = this.choosePointForSlope( slope, xRange, yRange );
          challenges.add( new MakeTheEquation( "2 of 2 required slopes, random choice of point-slope", Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
        }
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists, yInterceptListIndices );
        challenges.add( new GraphTheLine( "2 of 2 required y-intercepts", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists, slopeBinIndices );
        var point = this.choosePointForSlope( slope, xRange, yRange );
        challenges.add( new GraphTheLine( "3 of 3 required slopes", Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
      }
      {
        var positiveSlopes = ChallengeFactory2.createPositiveFractionalSlopes();
        positiveSlopes.add( new Fraction( 2, 1 ) );
        positiveSlopes.add( new Fraction( 3, 1 ) );
        positiveSlopes.add( new Fraction( 4, 1 ) );
        positiveSlopes.add( new Fraction( 5, 1 ) );
        var slope = fractionChooser.choose( positiveSlopes );
        var point = this.choosePointForSlopeInversion( slope, xRange, yRange );
        if ( equationFormChooser.choose( equationForms ) == this.SLOPE_INTERCEPT ) {
          challenges.add( new GraphTheLine( "slope-intercept because Make-the-Equation uses point-slope, force slope inversion", Line.createSlopeIntercept( this.numerator, this.denominator, point.getY() ), this.SLOPE_INTERCEPT, this.TWO_POINTS, xRange, yRange ) );
        }
        else {
          challenges.add( new GraphTheLine( "point-slope because Make-the-Equation uses slope-intercept, force slope inversion", Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, this.TWO_POINTS, xRange, yRange ) );
        }
      }
      Collections.shuffle( challenges );
      return challenges;
    },


  } );

} );

