// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var ArrayList = require( 'ArrayList' );
  var Collections = require( 'Collections' );
  var IntegerRange = require( 'IntegerRange' );
  var Line = require( 'Line' );

//Class declaration for ChallengeFactory6
  return inherit( ChallengeFactory, ChallengeFactory6, {

    /**
     * Creates challenges for this game level.
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     * @return list of challenges
     */
    createChallenges: function( xRange, yRange ) {
      var challenges = new ArrayList();
      var yIntercepts = this.rangeToList( new IntegerRange( -10, 10 ) );
      {
        var yIntercept = integerChooser.choose( yIntercepts );
        challenges.add( new PlaceThePoints( "slope=0", Line.createSlopeIntercept( 0, 1, yIntercept ), this.SLOPE_INTERCEPT, xRange, yRange ) );
      }
      ChallengeFactory5.addPlaceThePointsChallenges( challenges, integerChooser, xRange, yRange );
      {
        var equationForms = new ArrayList()
        {

          {
            this.add( this.SLOPE_INTERCEPT );
            this.add( this.SLOPE_INTERCEPT );
            this.add( this.POINT_SLOPE );
            this.add( this.POINT_SLOPE );
          }
        ,

        }
        ;
        assert( equationForms.size() == 4 );
        for ( var i = 0; i < 3; i++ ) {
          var equationForm = equationFormChooser.choose( equationForms );
          var range = new IntegerRange( -7, 7 );
          var xList = this.rangeToList( range );
          var yList = this.rangeToList( range );
          var x1 = 0;
          var y1 = integerChooser.choose( yList );
          var x2 = integerChooser.choose( xList );
          if ( x2 == x1 ) {
            x2 = integerChooser.choose( xList );
          }
          var y2 = integerChooser.choose( yList );
          var slope = (y2 - y1) / (x2 - x1);
          if ( slope == 1 || slope == -1 ) {
            y2 = integerChooser.choose( yList );
          }
          var line = new Line( x1, y1, x2, y2, this.BLACK );
          if ( equationForm == this.SLOPE_INTERCEPT ) {
            challenges.add( new GraphTheLine( "slope-intercept, Graph-the-Line " + i, line, equationForm, this.POINT_SLOPE, xRange, yRange ) );
          }
          else {
            challenges.add( new GraphTheLine( "point-slope, Graph-the-Line " + i, line, equationForm, this.SLOPE_INTERCEPT, xRange, yRange ) );
          }
        }
      }
      Collections.shuffle( challenges );
      return challenges;
    },


  } );

} );

