// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var ArrayList = require( 'ArrayList' );
  var IntegerRange = require( 'IntegerRange' );
  var Line = require( 'Line' );

//Class declaration for ChallengeFactoryHardCoded

  /**
   * Constructor for ChallengeFactoryHardCoded

   */
  function ChallengeFactoryHardCoded() {
  }

  return inherit( Object, ChallengeFactoryHardCoded, {

    /**
     * Creates hard-coded challenges, for development testing.
     * @param level the game level
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     * @return list of challenges
     */
    createChallenges: function( level, xRange, yRange ) {
      var challenges = new ArrayList();
      switch( level ) {
        case 1:
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createSlopeIntercept( 1, 1, -2 ), this.SLOPE_INTERCEPT, this.INTERCEPT, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createSlopeIntercept( 5, 1, 1 ), this.SLOPE_INTERCEPT, this.SLOPE, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createSlopeIntercept( 4, 2, 3 ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createSlopeIntercept( 3, 3, -3 ), this.SLOPE_INTERCEPT, this.TWO_POINTS, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createSlopeIntercept( 3, 3, -3 ), this.SLOPE_INTERCEPT, this.POINT_SLOPE, xRange, yRange ) );
          break;
        case 2:
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createPointSlope( 2, 1, 1, 2 ), this.POINT_SLOPE, this.SLOPE, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createPointSlope( 1, -3, 1, 3 ), this.POINT_SLOPE, this.POINT, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createPointSlope( -2, 1, -4, 3 ), this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createPointSlope( 5, 4, 3, 2 ), this.POINT_SLOPE, this.TWO_POINTS, xRange, yRange ) );
          challenges.add( new GraphTheLine( DESCRIPTION, Line.createSlopeIntercept( 4, 2, 3 ), this.POINT_SLOPE, this.SLOPE_INTERCEPT, xRange, yRange ) );
          break;
        case 3:
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createSlopeIntercept( 1, 1, -2 ), this.SLOPE_INTERCEPT, this.INTERCEPT, xRange, yRange ) );
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createSlopeIntercept( 5, 1, 1 ), this.SLOPE_INTERCEPT, this.SLOPE, xRange, yRange ) );
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createSlopeIntercept( 4, 2, 3 ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createSlopeIntercept( 3, 3, -3 ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
          break;
        case 4:
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createPointSlope( 2, 1, 1, 2 ), this.POINT_SLOPE, this.SLOPE, xRange, yRange ) );
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createPointSlope( 1, -3, 1, 3 ), this.POINT_SLOPE, this.POINT, xRange, yRange ) );
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createPointSlope( -2, 1, -4, 3 ), this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
          challenges.add( new MakeTheEquation( DESCRIPTION, Line.createPointSlope( 5, 4, 3, 2 ), this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
          break;
        case 5:
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createSlopeIntercept( 1, 1, -2 ), this.SLOPE_INTERCEPT, xRange, yRange ) );
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createSlopeIntercept( 5, 1, 1 ), this.SLOPE_INTERCEPT, xRange, yRange ) );
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createSlopeIntercept( 4, 2, 3 ), this.SLOPE_INTERCEPT, xRange, yRange ) );
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createSlopeIntercept( 3, 3, -3 ), this.SLOPE_INTERCEPT, xRange, yRange ) );
          break;
        case 6:
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createPointSlope( 2, 1, 1, 2 ), this.POINT_SLOPE, xRange, yRange ) );
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createPointSlope( 1, -3, 1, 3 ), this.POINT_SLOPE, xRange, yRange ) );
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createPointSlope( -2, 1, -4, 3 ), this.POINT_SLOPE, xRange, yRange ) );
          challenges.add( new PlaceThePoints( DESCRIPTION, Line.createPointSlope( 5, 4, 3, 2 ), this.POINT_SLOPE, xRange, yRange ) );
          break;
        default:
          throw new IllegalArgumentException( "unsupported level: " + level );
      }
      return challenges;
    },


  } );

} );

