// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var Vector2D = require( 'Vector2D' );
  var IntegerRange = require( 'IntegerRange' );
  var Strings = require( 'Strings' );
  var Line = require( 'Line' );
  var ChallengeNode = require( 'ChallengeNode' );
  var GraphTheLineNode = require( 'GraphTheLineNode' );
  var PDimension = require( 'PDimension' );

//Class declaration for GraphTheLine

  /**
   * Constructor
   * @param description brief description of the challenge, visible in dev versions
   * @param answer the correct answer
   * @param equationForm see EquationForm
   * @param manipulationMode see ManipulationMode
   * @param xRange range of the graph's x axis
   * @param yRange range of the graph's y axis
   */
  /**
   * Constructor for GraphTheLine
   * @param {String} description
   * @param {Line} answer
   * @param {EquationForm} equationForm
   * @param {ManipulationMode} manipulationMode
   * @param {IntegerRange} xRange
   * @param {IntegerRange} yRange
   */
  function GraphTheLine( description, answer, equationForm, manipulationMode, xRange, yRange ) {
    super( this.createTitle( this.GRAPH_THE_LINE, manipulationMode ), description, answer, equationForm, manipulationMode, xRange, yRange, new Point2D.Double( 735, 300 ), new Vector2D( xRange.getMin() + (0.65 * xRange.getLength()), yRange.getMin() - 1 ), new Vector2D( xRange.getMin() + (0.95 * xRange.getLength()), yRange.getMin() - 4 ) );
  }

  return inherit( Challenge, GraphTheLine, {

    updateGraphLines: function() {
      this.lines.clear();
      if ( guess.get() != null ) {
        this.lines.add( guess.get() );
      }
      if ( answerVisible ) {
        this.lines.add( answer );
      }
    },


    createView: function( model, challengeSize, audioPlayer ) {
      return new GraphTheLineNode( this, model, challengeSize, audioPlayer );
    },


  } );

} );

