// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var ArrayList = require( 'ArrayList' );
  var GameSettings = require( 'GameSettings' );
  var GameTimer = require( 'GameTimer' );
  var PhetApplication = require( 'PhetApplication' );
  var Property = require( 'Property' );
  var IntegerRange = require( 'IntegerRange' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var LoggingUtils = require( 'LoggingUtils' );
  var LGConstants = require( 'LGConstants' );
  var Line = require( 'Line' );

//Class declaration for LineGameModel

  /**
   * Constructor for LineGameModel

   */
  function LineGameModel() {
    settings = new GameSettings( LEVELS_RANGE, true, true );
    timer = new GameTimer();
    results = new GameResults( LEVELS_RANGE );
    challenge = new Property( DUMMY_CHALLENGE );
    phase = new Property( this.SETTINGS )
    {

      set
    :
      function( phase ) {
        if ( phase == this.SETTINGS ) {
          state.set( this.NONE );
          timer.stop();
        }
        else if ( phase == this.PLAY ) {
          this.initChallenges();
          state.set( this.FIRST_CHECK );
          this.score.set( 0 );
          timer.start();
        }
        else if ( phase == this.RESULTS ) {
          state.set( this.NONE );
          timer.stop();
          this.updateBestTime();
        }
        else {
          throw new UnsupportedOperationException( "unsupported game phase = " + phase );
        }
        super.set( phase );
      }
    }
    ;
    this.initChallenges();
    state = new Property( this.NONE )
    {

      {
        this.addObserver( new VoidFunction1()
        {

          apply: function( state ) {
            if ( state == this.FIRST_CHECK ) {
              if ( challengeIndex == challenges.size() ) {
                phase.set( this.RESULTS );
              }
              else {
                challenge.set( challenges.get( challengeIndex ) );
                challengeIndex++;
              }
            }
            else if ( state == this.NEXT ) {
              challenge.get().setAnswerVisible( true );
            }
          }
        }
      )
        ;
      }
    }
    ;
  }

  return inherit( Object, LineGameModel, {

    initChallenges: function() {
      challengeIndex = 0;
      if ( USE_HARD_CODED_CHALLENGES ) {
        LOGGER.info( "hard-coded challenges are enabled" );
        challenges = ChallengeFactoryHardCoded.createChallenges( this.level.get(), this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
      }
      else {
        switch( this.level.get() ) {
          case 1:
            challenges = factory1.createChallenges( this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
            break;
          case 2:
            challenges = factory2.createChallenges( this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
            break;
          case 3:
            challenges = factory3.createChallenges( this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
            break;
          case 4:
            challenges = factory4.createChallenges( this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
            break;
          case 5:
            challenges = factory5.createChallenges( this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
            break;
          case 6:
            challenges = factory6.createChallenges( this.X_AXIS_RANGE, this.Y_AXIS_RANGE );
            break;
          default:
            throw new IllegalArgumentException( "unsupported level: " + this.level.get() );
        }
      }
    },


    isPerfectScore: function() {
      return this.score.get() == this.getPerfectScore();
    },


    getPerfectScore: function() {
      return challenges.size() * this.computePoints( 1 );
    },


    updateBestTime: function() {
      assert( !timer.isRunning() );
      if ( this.timerEnabled.get() && this.score.get() == this.getPerfectScore() ) {
        results.updateBestTime( this.level.get(), this.time.get() );
      }
    },


    computePoints: function( attempts ) {
      return Math.max( 0, MAX_POINTS_PER_CHALLENGE - attempts + 1 );
    },


    /**
     * Skips the current challenge.
     * This is a developer feature.
     * Score and best times are meaningless after using this.
     */
    skipCurrentChallenge: function() {
      assert( PhetApplication.getInstance().isDeveloperControlsEnabled() );
      state.set( this.NEXT );
      state.set( this.FIRST_CHECK );
    },


    /**
     * Replays the current challenge.
     * This is a developer feature.
     * Score and best times are meaningless after using this.
     */
    replayCurrentChallenge: function() {
      assert( PhetApplication.getInstance().isDeveloperControlsEnabled() );
      challenge.get().reset();
      challengeIndex--;
      challenge.set( DUMMY_CHALLENGE );
      state.set( this.FIRST_CHECK );
    },


  } );

} );

