// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var Vector2D = require( 'Vector2D' );
  var Property = require( 'Property' );
  var IntegerRange = require( 'IntegerRange' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var Line = require( 'Line' );
  var LineGameConstants = require( 'LineGameConstants' );
  var ChallengeNode = require( 'ChallengeNode' );
  var PlaceThePointsNode = require( 'PlaceThePointsNode' );
  var PDimension = require( 'PDimension' );

//Class declaration for PlaceThePoints

  /**
   * Constructor
   * @param description brief description of the challenge, visible in dev versions
   * @param answer the correct answer
   * @param equationForm see EquationForm
   * @param xRange range of the graph's x axis
   * @param yRange range of the graph's y axis
   */
  /**
   * Constructor for PlaceThePoints
   * @param {String} description
   * @param {Line} answer
   * @param {EquationForm} equationForm
   * @param {IntegerRange} xRange
   * @param {IntegerRange} yRange
   */
  function PlaceThePoints( description, answer, equationForm, xRange, yRange ) {
    super( description, answer, equationForm, this.THREE_POINTS, xRange, yRange );
    this.p1 = new Property( new Vector2D( -3, 2 ) );
    this.p2 = new Property( new Vector2D( 0, 0 ) );
    this.p3 = new Property( new Vector2D( 3, 2 ) );
    var pointObserver = new RichSimpleObserver()
    {

      update: function() {
        var line = new Line( this.x, this.y, this.x, this.y, this.GUESS_COLOR );
        if ( line.onLine( p3.get() ) ) {
          guess.set( line );
          if ( this.isCorrect() ) {
            guess.set( answer );
          }
        }
        else {
          guess.set( null );
        }
      }
    }
    ;
    pointObserver.observe( p1, p2, p3 );
  }

  return inherit( GraphTheLine, PlaceThePoints, {

    reset: function() {
      super.reset();
      p1.reset();
      p2.reset();
      p3.reset();
    },


    createView: function( model, challengeSize, audioPlayer ) {
      return new PlaceThePointsNode( this, model, challengeSize, audioPlayer );
    },


  } );

} );

