// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var ArrayList = require( 'ArrayList' );
  var List = require( 'List' );
  var Random = require( 'Random' );
  var IntegerRange = require( 'IntegerRange' );

//Class declaration for RandomChooser

  /**
   * Constructor for RandomChooser
   * @param {Random} random
   */
  function RandomChooser( random ) {
    this.random = random;
  }

  return inherit( Object, RandomChooser, {

    /**
     *  Chooses a value from a list, removes the value from the list.
     *  @param list the list from which the value will be chosen
     *  @return a value
     */
    choose: function( list ) {
      var index = this.randomIndex( list );
      var value = list.get( index );
      list.remove( index );
      return value;
    },


    /**
     * Chooses a value from a list of lists, removes it from the list in which it was found.
     * Use this when values are organized into 2 or more sets.
     *
     * @param lists lists from which the value may be chosen
     * @return a value from one of the lists
     */
    chooseFromLists: function( lists ) {
      return this.chooseFromLists( lists, ChallengeFactory.rangeToList( new IntegerRange( 0, lists.size() - 1 ) ) );
    },


    /**
     * Chooses a value from a list of lists, removes it from the list in which it was found, removes the list from the listIndices.
     * Use this when values are organized into 2 or more sets, and a value should be chosen from each set.
     * By removing an index from listIndices, one of the lists is excluded from further consideration.
     *
     * @param lists lists from which the value may be chosen
     * @param listIndices indices of the lists that will be considered when choosing a value
     * @return a value from one of the lists
     */
    chooseFromLists: function( lists, listIndices ) {
      var index = this.randomIndex( listIndices );
      var bin = lists.get( listIndices.get( index ) );
      listIndices.remove( index );
      return this.choose( bin );
    },


    randomIndex: function( list ) {
      return random.nextInt( list.size() );
    },


  } );

} );

