// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Point2D = require( 'Point2D' );
  var PhetApplication = require( 'PhetApplication' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var Line = require( 'Line' );
  var GraphNode = require( 'GraphNode' );
  var LineNode = require( 'LineNode' );
  var PlottedPointNode = require( 'PlottedPointNode' );
  var SlopeToolNode = require( 'SlopeToolNode' );
  var LineGameConstants = require( 'LineGameConstants' );
  var Challenge = require( 'Challenge' );
  var PNode = require( 'PNode' );
  var PComposite = require( 'PComposite' );

//Class declaration for ChallengeGraphNode

  /**
   * Constructor for ChallengeGraphNode
   * @param {Challenge} challenge
   * @param {boolean} slopeToolEnabled
   */
  function ChallengeGraphNode( challenge, slopeToolEnabled ) {
    super( this.graph, this.mvt );
    if ( PhetApplication.getInstance().isDeveloperControlsEnabled() ) {
      this.addChild( new LineNode( this.answer.withColor( new Color( 0, 0, 0, 25 ) ), this.graph, this.mvt ) );
    }
    var pointDiameter = this.mvt.modelToViewDeltaX( this.POINT_DIAMETER );
    {
      answerParentNode = new PComposite();
      var answerNode = new LineNode( this.answer, this.graph, this.mvt );
      answerParentNode.addChild( answerNode );
      answerPointNode = new PlottedPointNode( pointDiameter, this.color );
      answerParentNode.addChild( answerPointNode );
      answerPointNode.setOffset( this.mvt.modelToView( new Point2D.Double( this.x1, this.y1 ) ) );
    }
    guessParentNode = new PComposite();
    slopeToolNode = slopeToolEnabled ? new SlopeToolNode( this.guess, this.mvt ) : new PNode();
    this.addChild( guessParentNode );
    this.addChild( answerParentNode );
    this.addChild( slopeToolNode );
    this.guess.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        guessParentNode.removeAllChildren();
        guessPointNode = null;
        if ( line != null ) {
          guessParentNode.addChild( new LineNode( line, this.graph, this.mvt ) );
          guessPointNode = new PlottedPointNode( pointDiameter, this.color );
          guessPointNode.setVisible( guessPointVisible );
          guessParentNode.addChild( guessPointNode );
          guessPointNode.setOffset( this.mvt.modelToView( this.x1, this.y1 ) );
        }
      }
    }
  )
    ;
    this.setAnswerVisible( false );
    this.setAnswerPointVisible( false );
    this.setGuessVisible( false );
    this.setGuessPointVisible( false );
    this.setSlopeToolVisible( false );
  }

  return inherit( GraphNode, ChallengeGraphNode, {

    setAnswerVisible: function( visible ) {
      answerParentNode.setVisible( visible );
    },


    setGuessVisible: function( visible ) {
      guessParentNode.setVisible( visible );
    },


    setAnswerPointVisible: function( visible ) {
      answerPointNode.setVisible( visible );
    },


    setGuessPointVisible: function( visible ) {
      guessPointVisible = visible;
      if ( guessPointNode != null ) {
        guessPointNode.setVisible( guessPointVisible );
      }
    },


    setSlopeToolVisible: function( visible ) {
      slopeToolNode.setVisible( visible );
    },


  } );

} );

