// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var BasicStroke = require( 'BasicStroke' );
  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var RoundRectangle2D = require( 'RoundRectangle2D' );
  var PhetFont = require( 'PhetFont' );
  var ZeroOffsetNode = require( 'ZeroOffsetNode' );
  var Images = require( 'Images' );
  var PNode = require( 'PNode' );
  var PImage = require( 'PImage' );
  var PPath = require( 'PPath' );
  var PText = require( 'PText' );
  var PDimension = require( 'PDimension' );

//Class declaration for EquationBoxNode

  /**
   * Constructor
   * @param title title that appears in upper-left corner of box
   * @param titleColor color of the title
   * @param boxSize size of the box
   * @param equationNode the equation that's in the box
   */
  /**
   * Constructor for EquationBoxNode
   * @param {String} title
   * @param {Color} titleColor
   * @param {PDimension} boxSize
   * @param {PNode} equationNode
   */
  function EquationBoxNode( title, titleColor, boxSize, equationNode ) {
    var titleNode = new PText( title );
    titleNode.setTextPaint( titleColor );
    titleNode.setFont( new PhetFont( this.BOLD, 24 ) );
    var maxTitleWidth = boxSize.getWidth() - (2 * X_MARGIN);
    if ( titleNode.getFullBoundsReference().getWidth() > maxTitleWidth ) {
      titleNode.scale( maxTitleWidth / titleNode.getFullBoundsReference().getWidth() );
    }
    var boxNode = new PPath( new RoundRectangle2D.Double( 0, 0, boxSize.getWidth(), boxSize.getHeight(), 20, 20 ) );
    boxNode.setStroke( new BasicStroke( 1
    f
  ))
    ;
    boxNode.setStrokePaint( this.BLACK );
    boxNode.setPaint( new Color( 238, 238, 238 ) );
    correctIconNode = new PImage( this.CHECK_MARK );
    incorrectIconNode = new PImage( this.X_MARK );
    var equationWrapperNode = new ZeroOffsetNode( equationNode );
    this.addChild( boxNode );
    this.addChild( titleNode );
    this.addChild( equationWrapperNode );
    this.addChild( correctIconNode );
    this.addChild( incorrectIconNode );
    {
      titleNode.setOffset( X_MARGIN, Y_MARGIN );
      var equationCenterY = titleNode.getFullBoundsReference().getMaxY() + ((boxSize.getHeight() - titleNode.getFullBoundsReference().getMaxY()) / 2);
      equationWrapperNode.setOffset( X_MARGIN, equationCenterY - (equationWrapperNode.getFullBoundsReference().getHeight() / 2) );
      var iconXMargin = 10;
      var iconYMargin = 5;
      correctIconNode.setOffset( boxNode.getFullBoundsReference().getMaxX() - correctIconNode.getFullBoundsReference().getWidth() - iconXMargin, boxNode.getFullBoundsReference().getMinY() + iconYMargin );
      incorrectIconNode.setOffset( boxNode.getFullBoundsReference().getMaxX() - incorrectIconNode.getFullBoundsReference().getWidth() - iconXMargin, boxNode.getFullBoundsReference().getMinY() + iconYMargin );
    }
    correctIconNode.setVisible( false );
    incorrectIconNode.setVisible( false );
  }

  return inherit( PNode, EquationBoxNode, {

    setCorrectIconVisible: function( visible ) {
      correctIconNode.setVisible( visible );
    },


    setIncorrectIconVisible: function( visible ) {
      incorrectIconNode.setVisible( visible );
    },


  } );

} );

