// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var SimpleObserver = require( 'SimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var CenteredStageCanvas = require( 'CenteredStageCanvas' );
  var LGColors = require( 'LGColors' );
  var GamePhase = require( 'GamePhase' );
  var LineGameModel = require( 'LineGameModel' );
  var PNode = require( 'PNode' );
  var PBounds = require( 'PBounds' );

//Class declaration for LineGameCanvas

  /**
   * Constructor for LineGameCanvas
   * @param {LineGameModel} model
   */
  function LineGameCanvas( model ) {
    this.setBackground( this.CANVAS );
    audioPlayer = new GameAudioPlayer( this.soundEnabled.get() );
    this.soundEnabled.addObserver( new SimpleObserver()
    {

      update: function() {
        audioPlayer.setEnabled( this.soundEnabled.get() );
      }
    }
  )
    ;
    settingsNode = new SettingsNode( model, this.getStageSize() );
    playNode = new PlayNode( model, this.getStageSize(), audioPlayer );
    resultsNode = new ResultsNode( model, this.getStageSize() );
    {
      this.addChild( resultsNode );
      this.addChild( playNode );
      this.addChild( settingsNode );
    }
    this.phase.addObserver( new VoidFunction1()
    {

      apply: function( gamePhase ) {
        settingsNode.setVisible( gamePhase == this.SETTINGS );
        playNode.setVisible( gamePhase == this.PLAY );
        resultsNode.setVisible( gamePhase == this.RESULTS );
        if ( gamePhase == this.RESULTS ) {
          if ( this.score.get() == model.getPerfectScore() ) {
            audioPlayer.gameOverPerfectScore();
          }
          else {
            audioPlayer.gameOverImperfectScore();
          }
        }
      }
    }
  )
    ;
  }

  return inherit( CenteredStageCanvas, LineGameCanvas, {

    updateLayout: function() {
      var worldBounds = this.getWorldBounds();
      if ( worldBounds.getWidth() > 0 && worldBounds.getHeight() > 0 ) {
        resultsNode.setRewardBounds( worldBounds );
      }
    },


    isRewardRunning: function() {
      return resultsNode.isRewardRunning();
    },


    setRewardRunning: function( running ) {
      resultsNode.setRewardRunning( running );
    },


  } );

} );

