// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var Challenge = require( 'Challenge' );
  var LineGameModel = require( 'LineGameModel' );
  var PlaceThePoints = require( 'PlaceThePoints' );
  var PlayState = require( 'PlayState' );
  var PDimension = require( 'PDimension' );

//Class declaration for PlaceThePointsNode

  /**
   * Constructor
   *
   * @param model         the game model
   * @param challengeSize dimensions of the view rectangle that is available for rendering the challenge
   * @param audioPlayer   the audio player, for providing audio feedback during game play
   * @param challenge the challenge
   */
  /**
   * Constructor for PlaceThePointsNode
   * @param {PlaceThePoints} challenge
   * @param {LineGameModel} model
   * @param {PDimension} challengeSize
   * @param {GameAudioPlayer} audioPlayer
   */
  function PlaceThePointsNode( challenge, model, challengeSize, audioPlayer ) {
    super( challenge, model, challengeSize, audioPlayer );
    this.state.addObserver( new VoidFunction1()
    {

      apply: function( state ) {
        graphNode.setGuessVisible( !challenge.isCorrect() && (state == this.TRY_AGAIN || state == this.NEXT) );
        graphNode.setAnswerPointVisible( state == this.NEXT && !challenge.isCorrect() );
        graphNode.setGuessPointVisible( false );
      }
    }
  )
    ;
  }

  return inherit( GraphTheLineNode, PlaceThePointsNode, {

    createGraphNode: function( challenge ) {
      assert( challenge instanceof PlaceThePoints );
      return new ThreePointsGraphNode( challenge );
    },


  } );

} );

