// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Dimension2D = require( 'Dimension2D' );
  var DecimalFormat = require( 'DecimalFormat' );
  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var GameScoreboardNode = require( 'GameScoreboardNode' );
  var GameScoreboardListener = require( 'GameScoreboardListener' );
  var SimpleObserver = require( 'SimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetPNode = require( 'PhetPNode' );
  var Challenge = require( 'Challenge' );
  var GamePhase = require( 'GamePhase' );
  var LineGameModel = require( 'LineGameModel' );
  var PNode = require( 'PNode' );
  var PDimension = require( 'PDimension' );

//Class declaration for PlayNode

  /**
   * Constructor for PlayNode
   * @param {LineGameModel} model
   * @param {Dimension2D} stageSize
   * @param {GameAudioPlayer} audioPlayer
   */
  function PlayNode( model, stageSize, audioPlayer ) {
    var scoreboardNode = new GameScoreboardNode( this.level.getMax(), model.getPerfectScore(), new DecimalFormat( "0" ) );
    scoreboardNode.setBackgroundWidth( stageSize.getWidth() - 150 );
    scoreboardNode.setOffset( (stageSize.getWidth() - scoreboardNode.getFullBoundsReference().getWidth()) / 2, stageSize.getHeight() - scoreboardNode.getFullBoundsReference().getHeight() - 10 );
    this.addChild( scoreboardNode );
    var challengeSize = new PDimension( stageSize.getWidth(), scoreboardNode.getFullBoundsReference().getMinY() );
    var challengeParent = new PNode();
    this.addChild( challengeParent );
    scoreboardNode.addGameScoreboardListener( new GameScoreboardListener()
    {

      newGamePressed: function() {
        this.phase.set( this.SETTINGS );
      }
    }
  )
    ;
    this.level.addObserver( new SimpleObserver()
    {

      update: function() {
        scoreboardNode.setLevel( this.level.get() );
      }
    }
  )
    ;
    this.score.addObserver( new SimpleObserver()
    {

      update: function() {
        scoreboardNode.setScore( this.score.get() );
      }
    }
  )
    ;
    this.timerEnabled.addObserver( new VoidFunction1()
    {

      apply: function( enabled ) {
        scoreboardNode.setTimerVisible( enabled );
      }
    }
  )
    ;
    this.time.addObserver( new VoidFunction1()
    {

      apply: function( t ) {
        scoreboardNode.setTime( t, this.results.getBestTime( this.level.get() ) );
      }
    }
  )
    ;
    this.challenge.addObserver( new VoidFunction1()
    {

      apply: function( challenge ) {
        challengeParent.removeAllChildren();
        challengeParent.addChild( challenge.createView( model, challengeSize, audioPlayer ) );
      }
    }
  )
    ;
  }

  return inherit( PhetPNode, PlayNode, {

  } );

} );

