// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Property = require( 'Property' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var DoubleRange = require( 'DoubleRange' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var LGColors = require( 'LGColors' );
  var UserComponents = require( 'UserComponents' );
  var Line = require( 'Line' );
  var LineManipulatorNode = require( 'LineManipulatorNode' );
  var SlopeDragHandler = require( 'SlopeDragHandler' );
  var X1Y1DragHandler = require( 'X1Y1DragHandler' );
  var LineGameConstants = require( 'LineGameConstants' );
  var Challenge = require( 'Challenge' );
  var ManipulationMode = require( 'ManipulationMode' );
  var PointSlopeParameterRange = require( 'PointSlopeParameterRange' );

//Class declaration for PointSlopeGraphNode

  /**
   * Constructor for PointSlopeGraphNode
   * @param {Challenge} challenge
   */
  function PointSlopeGraphNode( challenge ) {
    super( challenge, true );
    this.setGuessVisible( true );
    var pointSlopeParameterRange = new PointSlopeParameterRange();
    var x1Range = new Property( new DoubleRange( this.xRange ) );
    var y1Range = new Property( new DoubleRange( this.yRange ) );
    var riseRange = new Property( pointSlopeParameterRange.rise( this.guess.get(), this.graph ) );
    var runRange = new Property( pointSlopeParameterRange.run( this.guess.get(), this.graph ) );
    var manipulatorDiameter = this.mvt.modelToViewDeltaX( this.MANIPULATOR_DIAMETER );
    var pointManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.POINT_X1_Y1 );
    pointManipulatorNode.addInputEventListener( new X1Y1DragHandler( this.pointManipulator, this.sprite, pointManipulatorNode, this.mvt, this.guess, x1Range, y1Range, true ) );
    var slopeManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.SLOPE );
    slopeManipulatorNode.addInputEventListener( new SlopeDragHandler( this.slopeManipulator, this.sprite, slopeManipulatorNode, this.mvt, this.guess, riseRange, runRange ) );
    var pointIsVariable = (this.manipulationMode == this.POINT || this.manipulationMode == this.POINT_SLOPE);
    var slopeIsVariable = (this.manipulationMode == this.SLOPE || this.manipulationMode == this.POINT_SLOPE);
    if ( pointIsVariable ) {
      this.addChild( pointManipulatorNode );
    }
    if ( slopeIsVariable ) {
      this.addChild( slopeManipulatorNode );
    }
    this.guess.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        pointManipulatorNode.setOffset( this.mvt.modelToView( this.x1, this.y1 ) );
        slopeManipulatorNode.setOffset( this.mvt.modelToView( this.x2, this.y2 ) );
        if ( this.manipulationMode == this.POINT_SLOPE ) {
          var pointSlopeParameterRange = new PointSlopeParameterRange();
          x1Range.set( pointSlopeParameterRange.x1( line, this.graph ) );
          y1Range.set( pointSlopeParameterRange.y1( line, this.graph ) );
          riseRange.set( pointSlopeParameterRange.rise( line, this.graph ) );
          runRange.set( pointSlopeParameterRange.run( line, this.graph ) );
        }
      }
    }
  )
    ;
  }

  return inherit( ChallengeGraphNode, PointSlopeGraphNode, {

  } );

} );

