// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Dimension2D = require( 'Dimension2D' );
  var GameOverNode = require( 'GameOverNode' );
  var GameOverListener = require( 'GameOverListener' );
  var DefaultDecimalFormat = require( 'DefaultDecimalFormat' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetPNode = require( 'PhetPNode' );
  var LineGameConstants = require( 'LineGameConstants' );
  var GamePhase = require( 'GamePhase' );
  var LineGameModel = require( 'LineGameModel' );
  var PBounds = require( 'PBounds' );

//Class declaration for ResultsNode

  /**
   * Constructor for ResultsNode
   * @param {LineGameModel} model
   * @param {Dimension2D} stageSize
   */
  function ResultsNode( model, stageSize ) {
    rewardNode = new RewardNode();
    this.phase.addObserver( new VoidFunction1()
    {

      apply: function( phase ) {
        if ( phase == this.RESULTS ) {
          if ( model.isPerfectScore() ) {
            rewardNode.setLevel( this.level.get() );
            this.addChild( rewardNode );
            this.setRewardRunning( true );
          }
          var gameOverNode = new GameOverNode( this.level.get(), this.score.get(), model.getPerfectScore(), new DefaultDecimalFormat( "0" ), this.time.get(), this.results.getBestTime( this.level.get() ), this.isNewBestTime, this.timerEnabled.get(), this.BUTTON_COLOR );
          gameOverNode.scale( 1.5 );
          this.addChild( gameOverNode );
          gameOverNode.setOffset( (stageSize.getWidth() - gameOverNode.getFullBoundsReference().getWidth()) / 2, (stageSize.getHeight() - gameOverNode.getFullBoundsReference().getHeight()) / 2 );
          gameOverNode.addGameOverListener( new GameOverListener()
          {

            newGamePressed: function() {
              this.phase.set( this.SETTINGS );
            }
          }
        )
          ;
        }
        else {
          this.removeAllChildren();
          this.setRewardRunning( false );
        }
      }
    }
  )
    ;
  }

  return inherit( PhetPNode, ResultsNode, {

    setRewardBounds: function( bounds ) {
      rewardNode.setBounds( bounds );
    },


    isRewardRunning: function() {
      return rewardNode.isRunning();
    },


    setRewardRunning: function( running ) {
      rewardNode.setRunning( running );
    },


  } );

} );

