// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var BorderLayout = require( 'BorderLayout' );
  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var Image = require( 'Image' );
  var ActionEvent = require( 'ActionEvent' );
  var ActionListener = require( 'ActionListener' );
  var Point2D = require( 'Point2D' );
  var Rectangle2D = require( 'Rectangle2D' );
  var ArrayList = require( 'ArrayList' );
  var BorderFactory = require( 'BorderFactory' );
  var ButtonGroup = require( 'ButtonGroup' );
  var JCheckBox = require( 'JCheckBox' );
  var JFrame = require( 'JFrame' );
  var JPanel = require( 'JPanel' );
  var JRadioButton = require( 'JRadioButton' );
  var WindowConstants = require( 'WindowConstants' );
  var EmptyBorder = require( 'EmptyBorder' );
  var TitledBorder = require( 'TitledBorder' );
  var ChangeEvent = require( 'ChangeEvent' );
  var ChangeListener = require( 'ChangeListener' );
  var PhetApplicationConfig = require( 'PhetApplicationConfig' );
  var Vector2D = require( 'Vector2D' );
  var ClockAdapter = require( 'ClockAdapter' );
  var ClockEvent = require( 'ClockEvent' );
  var ConstantDtClock = require( 'ConstantDtClock' );
  var PhetCommonResources = require( 'PhetCommonResources' );
  var IntegerRange = require( 'IntegerRange' );
  var VerticalLayoutPanel = require( 'VerticalLayoutPanel' );
  var LinearValueControl = require( 'LinearValueControl' );
  var BufferedImageUtils = require( 'BufferedImageUtils' );
  var PhetFont = require( 'PhetFont' );
  var PhetPCanvas = require( 'PhetPCanvas' );
  var CenteredStage = require( 'CenteredStage' );
  var PhetPNode = require( 'PhetPNode' );
  var FaceNode = require( 'FaceNode' );
  var PadBoundsNode = require( 'PadBoundsNode' );
  var LGColors = require( 'LGColors' );
  var Line = require( 'Line' );
  var Orientation = require( 'Orientation' );
  var GraphNode = require( 'GraphNode' );
  var PointToolNode = require( 'PointToolNode' );
  var PointSlopeEquationNode = require( 'PointSlopeEquationNode' );
  var SlopeInterceptEquationNode = require( 'SlopeInterceptEquationNode' );
  var PNode = require( 'PNode' );
  var PImage = require( 'PImage' );
  var PBounds = require( 'PBounds' );
  var PDimension = require( 'PDimension' );

//Class declaration for RewardNode

  /**
   * Constructor for RewardNode

   */
  function RewardNode() {
    this( new PBounds( 0, 0, 1024, 768 ), 150, 10, 40 );
  }

  /**
   * Constructor.
   *
   * @param bounds      images are constrained to motion within these bounds
   * @param population  how many images in the animation
   * @param motionDelta nominal motion delta, may be randomly adjusted for each specific image
   * @param clockDelay  large value causes slower animation
   */
  /**
   * Constructor for RewardNode
   * @param {PBounds} bounds
   * @param {int} population
   * @param {int} motionDelta
   * @param {int} clockDelay
   */
  function RewardNode( bounds, population, motionDelta, clockDelay ) {
    super();
    this.setPickable( false );
    this.setChildrenPickable( false );
    this.population = population;
    this.motionDelta = motionDelta;
    this.motionStrategy = new FallingMotionStrategy();
    this.clock = new ConstantDtClock( clockDelay, 1 );
    this.clock.pause();
    {
      graphImages = new ArrayList();
      equationImages = new ArrayList();
      pointToolImages = new ArrayList();
      smileyFaceImages = new ArrayList();
      phetLogoImages = new ArrayList();
      var colors = new Color[]
      { this.YELLOW, this.RED, this.ORANGE, this.MAGENTA, this.CYAN, this.GREEN }
      ;
      for ( var color :
      colors
    )
      {
        graphImages.add( BufferedImageUtils.toImage( GraphNode.createYEqualsXIcon( 50, color ) ) );
        graphImages.add( BufferedImageUtils.toImage( GraphNode.createYEqualsNegativeXIcon( 50, color ) ) );
        pointToolImages.add( this.createPointToolImage( color ) );
        smileyFaceImages.add( this.createSmileyFaceImage( color ) );
        for ( var i = 0; i < 5; i++ ) {
          equationImages.add( this.createEquationImage( color ) );
        }
      }
      var phetLogoNode = new PImage( PhetCommonResources.getImage( "logos/phet-logo-120x50.jpg" ) );
      phetLogoNode.scale( 1.0 );
      phetLogoImages.add( phetLogoNode.toImage() );
      imagePool = new ArrayList();
    }
    clock.addClockListener( new ClockAdapter()
    {

      clockTicked: function( clockEvent ) {
        this.step();
      }
    }
  )
    ;
    this.setBounds( bounds );
    this.setGraphsVisible( true );
    this.setEquationsVisible( true );
    this.setPointToolsVisible( true );
    this.setPhETLogosVisible( true );
    this.setSmileyFacesVisible( true );
  }

  return inherit( PhetPNode, RewardNode, {

    createEquationImage: function( color ) {
      var useSlopeInterceptForm = (Math.random() < 0.5);
      var font = new PhetFont( this.BOLD, 24 );
      var node;
      if ( useSlopeInterceptForm ) {
        node = new SlopeInterceptEquationNode( Line.createSlopeIntercept( this.getRandomNonZeroInteger( -20, 20 ), this.getRandomNonZeroInteger( -20, 20 ), this.getRandomNonZeroInteger( -20, 20 ) ), font, color );
      }
      else {
        node = new PointSlopeEquationNode( Line.createPointSlope( this.getRandomNonZeroInteger( -20, 20 ), this.getRandomNonZeroInteger( -20, 20 ), this.getRandomNonZeroInteger( -20, 20 ), this.getRandomNonZeroInteger( -20, 20 ) ), font, color );
      }
      return new PadBoundsNode( node ).toImage();
    },


    getRandomNonZeroInteger: function( min, max ) {
      var i = (min + (Math.random() * (max - min)));
      if ( i == 0 ) {
        i = 1;
      }
      return i;
    },


    createPointToolImage: function( color ) {
      var point = new Vector2D( Math.random() * 20, Math.random() * 20 );
      var pointToolNode = new PointToolNode( point, this.DOWN, color );
      pointToolNode.scale( 0.75 );
      return new PadBoundsNode( pointToolNode ).toImage();
    },


    createSmileyFaceImage: function( color ) {
      return new PadBoundsNode( new FaceNode( 40, color ) ).toImage();
    },


    /**
     * Sets the animation parameters based on game difficulty level.
     * Levels 1-5 each have one image; level 6 is all images.
     *
     * @param level the game level
     */
    setLevel: function( level ) {
      assert( level >= 1 && level <= 6 );
      this.setImagesVisible( equationImages, level == 1 || level == 6 );
      this.setImagesVisible( graphImages, level == 2 || level == 6 );
      this.setImagesVisible( pointToolImages, level == 3 || level == 6 );
      this.setImagesVisible( smileyFaceImages, level == 4 || level == 6 );
      this.setImagesVisible( phetLogoImages, level == 5 || level == 6 );
    },


    /**
     * When this node's bounds are changes, all images are repopulated
     * so that the images are distributed randomly throughout the entire bounds.
     *
     * @param bounds the bounds that the reward must fill
     */
    setBounds: function( bounds ) {
      if ( bounds.isEmpty() ) {
        throw new IllegalArgumentException( "bounds are empty" );
      }
      var boundsChanged = false;
      if ( !bounds.equals( this.getBounds() ) ) {
        boundsChanged = super.setBounds( bounds );
        this.updateImages( true );
      }
      return boundsChanged;
    },


    setClockDelay: function( delay ) {
      clock.setDelay( delay );
    },


    getClockDelay: function() {
      return clock.getDelay();
    },


    setPopulation: function( population ) {
      if ( !(population > 0) ) {
        throw new IllegalArgumentException( "population must be > 0: " + population );
      }
      if ( population != this.getPopulation() ) {
        this.population = population;
        this.updateImages( false );
      }
    },


    getPopulation: function() {
      return population;
    },


    setMotionDelta: function( motionDelta ) {
      if ( !(motionDelta > 0) ) {
        throw new IllegalArgumentException( "motionDelta must be > 0: " + motionDelta );
      }
      if ( motionDelta != this.getMotionDelta() ) {
        this.motionDelta = motionDelta;
        for ( var i = 0; i < this.getChildrenCount(); i++ ) {
          if ( this.getChild( i ) instanceof MovingImageNode ) {
            var delta = Math.max( 1, Math.random() * motionDelta );
            (this.getChild( i )).setMotionDelta( delta );
          }
        }
      }
    },


    getMotionDelta: function() {
      return motionDelta;
    },


    isGraphsVisible: function() {
      return this.isImagesVisible( graphImages );
    },


    setGraphsVisible: function( visible ) {
      this.setImagesVisible( graphImages, visible );
    },


    isEquationsVisible: function() {
      return this.isImagesVisible( equationImages );
    },


    setEquationsVisible: function( visible ) {
      this.setImagesVisible( equationImages, visible );
    },


    isPointToolsVisible: function() {
      return this.isImagesVisible( pointToolImages );
    },


    setPointToolsVisible: function( visible ) {
      this.setImagesVisible( pointToolImages, visible );
    },


    isPhETLogosVisible: function() {
      return this.isImagesVisible( phetLogoImages );
    },


    setPhETLogosVisible: function( visible ) {
      this.setImagesVisible( phetLogoImages, visible );
    },


    isSmileyFacesVisible: function() {
      return this.isImagesVisible( smileyFaceImages );
    },


    setSmileyFacesVisible: function( visible ) {
      this.setImagesVisible( smileyFaceImages, visible );
    },


    setImagesVisible: function( images, visible ) {
      if ( visible ) {
        imagePool.addAll( images );
      }
      else {
        imagePool.removeAll( images );
      }
      this.updateImages( true );
    },


    isImagesVisible: function( images ) {
      return imagePool.contains( images.get( 0 ) );
    },


    setMotionStrategy: function( motionStrategy ) {
      this.motionStrategy = motionStrategy;
    },


    updateImages: function( removeImages ) {
      if ( removeImages ) {
        this.removeAllChildren();
      }
      if ( imagePool.size() > 0 ) {
        if ( this.getChildrenCount() > population ) {
          while ( this.getChildrenCount() > population ) {
            this.removeChild( this.getChildrenCount() - 1 );
          }
        }
        else {
          while ( this.getChildrenCount() < population ) {
            this.addRandomNode();
          }
        }
      }
    },


    addRandomNode: function() {
      var delta = Math.max( 1, Math.random() * motionDelta );
      var index = (Math.random() * imagePool.size());
      var imageNode = new MovingImageNode( imagePool.get( index ), delta );
      this.addChild( imageNode );
      var bounds = this.getBoundsReference();
      var location = this.getRandomPoint( bounds );
      var x = location.getX() - (imageNode.getFullBoundsReference().getWidth() / 2);
      var y = location.getY() - (imageNode.getFullBoundsReference().getHeight() / 2);
      imageNode.setOffset( x, y );
    },


    getRandomPoint: function( bounds ) {
      var x = bounds.getX() + (Math.random() * bounds.getWidth());
      var y = bounds.getY() + (Math.random() * bounds.getHeight());
      return new Point2D.Double( x, y );
    },


    setVisible: function( visible ) {
      super.setVisible( visible );
      this.setRunning( visible );
    },


    setRunning: function( running ) {
      if ( running ) {
        clock.start();
      }
      else {
        clock.pause();
      }
    },


    isRunning: function() {
      return clock.isRunning();
    },


    step: function() {
      var bounds = this.getBoundsReference();
      for ( var i = 0; i < this.getChildrenCount(); i++ ) {
        if ( this.getChild( i ) instanceof MovingImageNode ) {
          motionStrategy.step( this.getChild( i ), bounds );
        }
      }
    },


    /**
     * This application was provided to team members so that they could experiment with settings.
     * During one stage of development, it was published to dev as flavor "test-game-reward".
     */
    main: function( args ) {
      var populationRange = new IntegerRange( 50, 1000, 500 );
      var motionDeltaRange = new IntegerRange( 1, 20, 5 );
      var clockDelayRange = new IntegerRange( 10, 200, 60 );
      var bounds = new PBounds( 0, 0, 100, 100 );
      var rewardNode = new RewardNode( bounds, populationRange.getDefault(), motionDeltaRange.getDefault(), clockDelayRange.getDefault() );
      rewardNode.setRunning( true );
      var canvas = new PhetPCanvas()
      {

        updateLayout: function() {
          var worldBounds = this.getWorldBounds();
          if ( worldBounds.getWidth() > 0 && worldBounds.getHeight() > 0 ) {
            rewardNode.setBounds( worldBounds );
          }
        }
      ,

      }
      ;
      var stageSize = new PDimension( 1008, 679 );
      canvas.setWorldTransformStrategy( new CenteredStage( canvas, stageSize ) );
      canvas.setBackground( this.CANVAS );
      canvas.addWorldChild( rewardNode );
      var jitteryMotionStrategy = new JitteryMotionStrategy();
      var fallingMotionStrategy = new FallingMotionStrategy();
      var jitteryMotionButton = new JRadioButton( "jittery" );
      jitteryMotionButton.addActionListener( new ActionListener()
      {

        actionPerformed: function( e ) {
          rewardNode.setMotionStrategy( jitteryMotionStrategy );
        }
      ,

      }
      )
      ;
      var fallingMotionButton = new JRadioButton( "falling" );
      fallingMotionButton.addActionListener( new ActionListener()
      {

        actionPerformed: function( e ) {
          rewardNode.setMotionStrategy( fallingMotionStrategy );
        }
      ,

      }
      )
      ;
      var motionGroup = new ButtonGroup();
      motionGroup.add( jitteryMotionButton );
      motionGroup.add( fallingMotionButton );
      rewardNode.setMotionStrategy( fallingMotionStrategy );
      fallingMotionButton.setSelected( true );
      var motionControlPanel = new VerticalLayoutPanel();
      motionControlPanel.setBorder( new TitledBorder( "Motion Strategy" ) );
      motionControlPanel.add( jitteryMotionButton );
      motionControlPanel.add( fallingMotionButton );
      var graphsCheckBox = new JCheckBox( "graphs" );
      graphsCheckBox.setSelected( rewardNode.isGraphsVisible() );
      graphsCheckBox.addActionListener( new ActionListener()
      {

        actionPerformed: function( e ) {
          rewardNode.setGraphsVisible( graphsCheckBox.isSelected() );
        }
      ,

      }
      )
      ;
      var equationsCheckBox = new JCheckBox( "equations" );
      equationsCheckBox.setSelected( rewardNode.isEquationsVisible() );
      equationsCheckBox.addActionListener( new ActionListener()
      {

        actionPerformed: function( e ) {
          rewardNode.setEquationsVisible( equationsCheckBox.isSelected() );
        }
      ,

      }
      )
      ;
      var pointToolsCheckBox = new JCheckBox( "point tools" );
      pointToolsCheckBox.setSelected( rewardNode.isPointToolsVisible() );
      pointToolsCheckBox.addActionListener( new ActionListener()
      {

        actionPerformed: function( e ) {
          rewardNode.setPointToolsVisible( pointToolsCheckBox.isSelected() );
        }
      ,

      }
      )
      ;
      var phetLogosCheckBox = new JCheckBox( "PhET logos" );
      phetLogosCheckBox.setSelected( rewardNode.isPhETLogosVisible() );
      phetLogosCheckBox.addActionListener( new ActionListener()
      {

        actionPerformed: function( e ) {
          rewardNode.setPhETLogosVisible( phetLogosCheckBox.isSelected() );
        }
      ,

      }
      )
      ;
      var smileyFacesCheckBox = new JCheckBox( "smiley faces" );
      smileyFacesCheckBox.setSelected( rewardNode.isSmileyFacesVisible() );
      smileyFacesCheckBox.addActionListener( new ActionListener()
      {

        actionPerformed: function( e ) {
          rewardNode.setSmileyFacesVisible( smileyFacesCheckBox.isSelected() );
        }
      ,

      }
      )
      ;
      var objectsControlPanel = new VerticalLayoutPanel();
      objectsControlPanel.setBorder( new TitledBorder( "Objects" ) );
      objectsControlPanel.add( graphsCheckBox );
      objectsControlPanel.add( equationsCheckBox );
      objectsControlPanel.add( pointToolsCheckBox );
      objectsControlPanel.add( phetLogosCheckBox );
      objectsControlPanel.add( smileyFacesCheckBox );
      var populationControl = new LinearValueControl( populationRange.getMin(), populationRange.getMax(), "population:", "##0", "" );
      populationControl.setBorder( BorderFactory.createEtchedBorder() );
      populationControl.setValue( populationRange.getDefault() );
      populationControl.setToolTipText( "how many images are in the play area" );
      populationControl.addChangeListener( new ChangeListener()
      {

        stateChanged: function( e ) {
          rewardNode.setPopulation( populationControl.getValue() );
        }
      ,

      }
      )
      ;
      var motionDeltaControl = new LinearValueControl( motionDeltaRange.getMin(), motionDeltaRange.getMax(), "motion delta:", "##0", "pixels" );
      motionDeltaControl.setBorder( BorderFactory.createEtchedBorder() );
      motionDeltaControl.setValue( motionDeltaRange.getDefault() );
      motionDeltaControl.setToolTipText( "how far images move each time the clock ticks" );
      motionDeltaControl.addChangeListener( new ChangeListener()
      {

        stateChanged: function( e ) {
          rewardNode.setMotionDelta( motionDeltaControl.getValue() );
        }
      ,

      }
      )
      ;
      var clockDelayControl = new LinearValueControl( clockDelayRange.getMin(), clockDelayRange.getMax(), "clock delay:", "##0", "ms" );
      clockDelayControl.setBorder( BorderFactory.createEtchedBorder() );
      clockDelayControl.setToolTipText( "how frequently the simulation clock ticks" );
      clockDelayControl.setValue( rewardNode.getClockDelay() );
      clockDelayControl.addChangeListener( new ChangeListener()
      {

        stateChanged: function( e ) {
          rewardNode.setClockDelay( clockDelayControl.getValue() );
        }
      ,

      }
      )
      ;
      var devControlPanel = new VerticalLayoutPanel();
      devControlPanel.add( motionControlPanel );
      devControlPanel.add( objectsControlPanel );
      devControlPanel.add( populationControl );
      devControlPanel.add( motionDeltaControl );
      devControlPanel.add( clockDelayControl );
      var controlPanel = new JPanel();
      controlPanel.setBorder( new EmptyBorder( 10, 10, 10, 10 ) );
      controlPanel.add( devControlPanel );
      var panel = new JPanel( new BorderLayout() );
      panel.add( canvas, this.CENTER );
      panel.add( controlPanel, this.EAST );
      var frame = new JFrame( "Game Reward test" );
      frame.setContentPane( panel );
      frame.setDefaultCloseOperation( this.EXIT_ON_CLOSE );
      this.DEFAULT_FRAME_SETUP.initialize( frame );
      frame.setVisible( true );
    },


  } );

} );

