// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Property = require( 'Property' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var DoubleRange = require( 'DoubleRange' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var LGColors = require( 'LGColors' );
  var UserComponents = require( 'UserComponents' );
  var Line = require( 'Line' );
  var LineManipulatorNode = require( 'LineManipulatorNode' );
  var SlopeDragHandler = require( 'SlopeDragHandler' );
  var X1Y1DragHandler = require( 'X1Y1DragHandler' );
  var LineGameConstants = require( 'LineGameConstants' );
  var Challenge = require( 'Challenge' );
  var ManipulationMode = require( 'ManipulationMode' );
  var SlopeInterceptParameterRange = require( 'SlopeInterceptParameterRange' );

//Class declaration for SlopeInterceptGraphNode

  /**
   * Constructor for SlopeInterceptGraphNode
   * @param {Challenge} challenge
   */
  function SlopeInterceptGraphNode( challenge ) {
    super( challenge, true );
    this.setGuessVisible( true );
    var parameterRange = new SlopeInterceptParameterRange();
    var riseRange = new Property( parameterRange.rise( this.guess.get(), this.graph ) );
    var runRange = new Property( parameterRange.run( this.guess.get(), this.graph ) );
    var y1Range = new Property( new DoubleRange( this.yRange ) );
    var manipulatorDiameter = this.mvt.modelToViewDeltaX( this.MANIPULATOR_DIAMETER );
    var slopeManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.SLOPE );
    slopeManipulatorNode.addInputEventListener( new SlopeDragHandler( this.slopeManipulator, this.sprite, slopeManipulatorNode, this.mvt, this.guess, riseRange, runRange ) );
    var interceptManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.INTERCEPT );
    interceptManipulatorNode.addInputEventListener( new X1Y1DragHandler( this.pointManipulator, this.sprite, interceptManipulatorNode, this.mvt, this.guess, new Property( new DoubleRange( 0, 0 ) ), y1Range, true ) );
    var slopeIsVariable = (this.manipulationMode == this.SLOPE || this.manipulationMode == this.SLOPE_INTERCEPT);
    var interceptIsVariable = (this.manipulationMode == this.INTERCEPT || this.manipulationMode == this.SLOPE_INTERCEPT);
    if ( interceptIsVariable ) {
      this.addChild( interceptManipulatorNode );
    }
    if ( slopeIsVariable ) {
      this.addChild( slopeManipulatorNode );
    }
    this.guess.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        slopeManipulatorNode.setOffset( this.mvt.modelToView( this.x2, this.y2 ) );
        interceptManipulatorNode.setOffset( this.mvt.modelToView( this.x1, this.y1 ) );
        if ( this.manipulationMode == this.SLOPE_INTERCEPT ) {
          var parameterRange = new SlopeInterceptParameterRange();
          riseRange.set( parameterRange.rise( line, this.graph ) );
          y1Range.set( parameterRange.y1( line, this.graph ) );
        }
      }
    }
  )
    ;
  }

  return inherit( ChallengeGraphNode, SlopeInterceptGraphNode, {

  } );

} );

