// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var ArrayList = require( 'ArrayList' );
  var Vector2D = require( 'Vector2D' );
  var Property = require( 'Property' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var DoubleRange = require( 'DoubleRange' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var LGColors = require( 'LGColors' );
  var UserComponents = require( 'UserComponents' );
  var LineManipulatorNode = require( 'LineManipulatorNode' );
  var PointDragHandler = require( 'PointDragHandler' );
  var LineGameConstants = require( 'LineGameConstants' );
  var PlaceThePoints = require( 'PlaceThePoints' );

//Class declaration for ThreePointsGraphNode

  /**
   * Constructor for ThreePointsGraphNode
   * @param {PlaceThePoints} challenge
   */
  function ThreePointsGraphNode( challenge ) {
    super( challenge, false );
    var manipulatorDiameter = this.mvt.modelToViewDeltaX( this.MANIPULATOR_DIAMETER );
    var p1ManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.POINT_1 );
    p1ManipulatorNode.addInputEventListener( new PointDragHandler( this.p1Manipulator, this.sprite, p1ManipulatorNode, this.mvt, this.p1, new ArrayList()
    {

      {
        this.add( this.p2 );
        this.add( this.p3 );
      }
    }
  ,
    new Property( new DoubleRange( this.xRange ) ), new Property( new DoubleRange( this.yRange ) )
  ))
    ;
    var p2ManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.POINT_2 );
    p2ManipulatorNode.addInputEventListener( new PointDragHandler( this.p2Manipulator, this.sprite, p2ManipulatorNode, this.mvt, this.p2, new ArrayList()
    {

      {
        this.add( this.p1 );
        this.add( this.p3 );
      }
    }
  ,
    new Property( new DoubleRange( this.xRange ) ), new Property( new DoubleRange( this.yRange ) )
  ))
    ;
    var p3ManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.POINT_3 );
    p3ManipulatorNode.addInputEventListener( new PointDragHandler( this.p3Manipulator, this.sprite, p3ManipulatorNode, this.mvt, this.p3, new ArrayList()
    {

      {
        this.add( this.p1 );
        this.add( this.p2 );
      }
    }
  ,
    new Property( new DoubleRange( this.xRange ) ), new Property( new DoubleRange( this.yRange ) )
  ))
    ;
    this.addChild( p1ManipulatorNode );
    this.addChild( p2ManipulatorNode );
    this.addChild( p3ManipulatorNode );
    var pointsObserver = new RichSimpleObserver()
    {

      update: function() {
        p1ManipulatorNode.setOffset( this.mvt.modelToView( this.x, this.y ) );
        p2ManipulatorNode.setOffset( this.mvt.modelToView( this.x, this.y ) );
        p3ManipulatorNode.setOffset( this.mvt.modelToView( this.x, this.y ) );
      }
    }
    ;
    pointsObserver.observe( this.p1, this.p2, this.p3 );
  }

  return inherit( ChallengeGraphNode, ThreePointsGraphNode, {

  } );

} );

