// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Property = require( 'Property' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var DoubleRange = require( 'DoubleRange' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var LGColors = require( 'LGColors' );
  var UserComponents = require( 'UserComponents' );
  var Line = require( 'Line' );
  var LineManipulatorNode = require( 'LineManipulatorNode' );
  var X1Y1DragHandler = require( 'X1Y1DragHandler' );
  var X2Y2DragHandler = require( 'X2Y2DragHandler' );
  var LineGameConstants = require( 'LineGameConstants' );
  var Challenge = require( 'Challenge' );

//Class declaration for TwoPointsGraphNode

  /**
   * Constructor for TwoPointsGraphNode
   * @param {Challenge} challenge
   */
  function TwoPointsGraphNode( challenge ) {
    super( challenge, true );
    this.setGuessVisible( true );
    var manipulatorDiameter = this.mvt.modelToViewDeltaX( this.MANIPULATOR_DIAMETER );
    var x1y1ManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.POINT_X1_Y1 );
    x1y1ManipulatorNode.addInputEventListener( new X1Y1DragHandler( this.x1y1Manipulator, this.sprite, x1y1ManipulatorNode, this.mvt, this.guess, new Property( new DoubleRange( this.xRange ) ), new Property( new DoubleRange( this.yRange ) ), false ) );
    var x2y2ManipulatorNode = new LineManipulatorNode( manipulatorDiameter, this.POINT_X2_Y2 );
    x2y2ManipulatorNode.addInputEventListener( new X2Y2DragHandler( this.x2y2Manipulator, this.sprite, x2y2ManipulatorNode, this.mvt, this.guess, new Property( new DoubleRange( this.xRange ) ), new Property( new DoubleRange( this.yRange ) ) ) );
    this.addChild( x1y1ManipulatorNode );
    this.addChild( x2y2ManipulatorNode );
    this.guess.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        x1y1ManipulatorNode.setOffset( this.mvt.modelToView( this.x1, this.y1 ) );
        x2y2ManipulatorNode.setOffset( this.mvt.modelToView( this.x2, this.y2 ) );
      }
    }
  )
    ;
  }

  return inherit( ChallengeGraphNode, TwoPointsGraphNode, {

  } );

} );

