// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Property = require( 'Property' );
  var DoubleRange = require( 'DoubleRange' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var LGColors = require( 'LGColors' );
  var Line = require( 'Line' );
  var LineFormsModel = require( 'LineFormsModel' );

//Class declaration for PointSlopeModel

  /**
   * Constructor for PointSlopeModel

   */
  function PointSlopeModel() {
    this( Line.createPointSlope( 1, 2, 3, 4, this.INTERACTIVE_LINE ), new PointSlopeParameterRange() );
  }

  /**
   * Constructor for PointSlopeModel
   * @param {Line} interactiveLine
   * @param {PointSlopeParameterRange} parameterRange
   */
  function PointSlopeModel( interactiveLine, parameterRange ) {
    super( interactiveLine );
    x1Range = new Property( new DoubleRange( this.xRange ) );
    y1Range = new Property( new DoubleRange( this.yRange ) );
    riseRange = new Property( new DoubleRange( this.yRange ) );
    runRange = new Property( new DoubleRange( this.xRange ) );
    this.interactiveLine.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        x1Range.set( parameterRange.x1( line, graph ) );
        y1Range.set( parameterRange.y1( line, graph ) );
        riseRange.set( parameterRange.rise( line, graph ) );
        runRange.set( parameterRange.run( line, graph ) );
      }
    }
  )
    ;
  }

  return inherit( LineFormsModel, PointSlopeModel, {

  } );

} );

