// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var LGColors = require( 'LGColors' );
  var UserComponents = require( 'UserComponents' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var LineFormsGraphNode = require( 'LineFormsGraphNode' );
  var LineFormsViewProperties = require( 'LineFormsViewProperties' );
  var LineNode = require( 'LineNode' );
  var LineManipulatorNode = require( 'LineManipulatorNode' );
  var SlopeDragHandler = require( 'SlopeDragHandler' );
  var X1Y1DragHandler = require( 'X1Y1DragHandler' );
  var PointSlopeModel = require( 'PointSlopeModel' );

//Class declaration for PointSlopeGraphNode

  /**
   * Constructor for PointSlopeGraphNode
   * @param {PointSlopeModel} model
   * @param {LineFormsViewProperties} viewProperties
   */
  function PointSlopeGraphNode( model, viewProperties ) {
    super( model, viewProperties );
    pointManipulatorNode = new LineManipulatorNode( this.getManipulatorDiameter(), this.POINT_X1_Y1 );
    pointManipulatorNode.addInputEventListener( new X1Y1DragHandler( this.pointManipulator, this.sprite, pointManipulatorNode, this.mvt, this.interactiveLine, this.x1Range, this.y1Range, true ) );
    slopeManipulatorNode = new LineManipulatorNode( this.getManipulatorDiameter(), this.SLOPE );
    slopeManipulatorNode.addInputEventListener( new SlopeDragHandler( this.slopeManipulator, this.sprite, slopeManipulatorNode, this.mvt, this.interactiveLine, this.riseRange, this.runRange ) );
    this.addChild( pointManipulatorNode );
    this.addChild( slopeManipulatorNode );
    this.interactiveLine.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        pointManipulatorNode.setOffset( this.mvt.modelToView( new Point2D.Double( this.x1, this.y1 ) ) );
        slopeManipulatorNode.setOffset( this.mvt.modelToView( new Point2D.Double( this.x1 + this.run, this.y1 + this.rise ) ) );
      }
    }
  )
    ;
    var visibilityObserver = new RichSimpleObserver()
    {

      update: function() {
        var visible = this.linesVisible.get() && this.interactiveLineVisible.get();
        pointManipulatorNode.setVisible( visible );
        slopeManipulatorNode.setVisible( visible );
      }
    }
    ;
    visibilityObserver.observe( this.linesVisible, this.interactiveLineVisible );
  }

  return inherit( LineFormsGraphNode, PointSlopeGraphNode, {

    createLineNode: function( line, graph, mvt ) {
      return new PointSlopeLineNode( line, graph, mvt );
    },


  } );

} );

