// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var LGColors = require( 'LGColors' );
  var UserComponents = require( 'UserComponents' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var LineFormsGraphNode = require( 'LineFormsGraphNode' );
  var LineFormsViewProperties = require( 'LineFormsViewProperties' );
  var LineNode = require( 'LineNode' );
  var LineManipulatorNode = require( 'LineManipulatorNode' );
  var X1Y1DragHandler = require( 'X1Y1DragHandler' );
  var X2Y2DragHandler = require( 'X2Y2DragHandler' );
  var SlopeModel = require( 'SlopeModel' );

//Class declaration for SlopeGraphNode

  /**
   * Constructor for SlopeGraphNode
   * @param {SlopeModel} model
   * @param {LineFormsViewProperties} viewProperties
   */
  function SlopeGraphNode( model, viewProperties ) {
    super( model, viewProperties );
    x1y1ManipulatorNode = new LineManipulatorNode( this.getManipulatorDiameter(), this.POINT_X1_Y1 );
    x1y1ManipulatorNode.addInputEventListener( new X1Y1DragHandler( this.x1y1Manipulator, this.sprite, x1y1ManipulatorNode, this.mvt, this.interactiveLine, this.x1Range, this.y1Range, false ) );
    x2y2ManipulatorNode = new LineManipulatorNode( this.getManipulatorDiameter(), this.POINT_X2_Y2 );
    x2y2ManipulatorNode.addInputEventListener( new X2Y2DragHandler( this.x2y2Manipulator, this.sprite, x2y2ManipulatorNode, this.mvt, this.interactiveLine, this.x2Range, this.y2Range ) );
    this.addChild( x1y1ManipulatorNode );
    this.addChild( x2y2ManipulatorNode );
    this.interactiveLine.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        x1y1ManipulatorNode.setOffset( this.mvt.modelToView( new Point2D.Double( this.x1, this.y1 ) ) );
        x2y2ManipulatorNode.setOffset( this.mvt.modelToView( new Point2D.Double( this.x2, this.y2 ) ) );
      }
    }
  )
    ;
    var visibilityObserver = new RichSimpleObserver()
    {

      update: function() {
        var visible = this.linesVisible.get() && this.interactiveLineVisible.get();
        x1y1ManipulatorNode.setVisible( visible );
        x2y2ManipulatorNode.setVisible( visible );
      }
    }
    ;
    visibilityObserver.observe( this.linesVisible, this.interactiveLineVisible );
  }

  return inherit( LineFormsGraphNode, SlopeGraphNode, {

    createLineNode: function( line, graph, mvt ) {
      return new SlopeLineNode( line, graph, mvt );
    },


  } );

} );

