// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var LGColors = require( 'LGColors' );
  var UserComponents = require( 'UserComponents' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var LineFormsGraphNode = require( 'LineFormsGraphNode' );
  var LineFormsViewProperties = require( 'LineFormsViewProperties' );
  var LineNode = require( 'LineNode' );
  var LineManipulatorNode = require( 'LineManipulatorNode' );
  var SlopeDragHandler = require( 'SlopeDragHandler' );
  var YInterceptDragHandler = require( 'YInterceptDragHandler' );
  var SlopeInterceptModel = require( 'SlopeInterceptModel' );

//Class declaration for SlopeInterceptGraphNode

  /**
   * Constructor for SlopeInterceptGraphNode
   * @param {SlopeInterceptModel} model
   * @param {LineFormsViewProperties} viewProperties
   */
  function SlopeInterceptGraphNode( model, viewProperties ) {
    super( model, viewProperties );
    slopeManipulatorNode = new LineManipulatorNode( this.getManipulatorDiameter(), this.SLOPE );
    slopeManipulatorNode.addInputEventListener( new SlopeDragHandler( this.slopeManipulator, this.sprite, slopeManipulatorNode, this.mvt, this.interactiveLine, this.riseRange, this.runRange ) );
    interceptManipulatorNode = new LineManipulatorNode( this.getManipulatorDiameter(), this.INTERCEPT );
    interceptManipulatorNode.addInputEventListener( new YInterceptDragHandler( this.interceptManipulator, this.sprite, interceptManipulatorNode, this.mvt, this.interactiveLine, this.y1Range ) );
    this.addChild( slopeManipulatorNode );
    this.addChild( interceptManipulatorNode );
    this.interactiveLine.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        slopeManipulatorNode.setOffset( this.mvt.modelToView( new Point2D.Double( this.x1 + this.run, this.y1 + this.rise ) ) );
        interceptManipulatorNode.setOffset( this.mvt.modelToView( new Point2D.Double( this.x1, this.y1 ) ) );
      }
    }
  )
    ;
    var visibilityObserver = new RichSimpleObserver()
    {

      update: function() {
        var visible = this.linesVisible.get() && this.interactiveLineVisible.get();
        slopeManipulatorNode.setVisible( visible );
        interceptManipulatorNode.setVisible( visible );
      }
    }
    ;
    visibilityObserver.observe( this.linesVisible, this.interactiveLineVisible );
  }

  return inherit( LineFormsGraphNode, SlopeInterceptGraphNode, {

    createLineNode: function( line, graph, mvt ) {
      return new SlopeInterceptLineNode( line, graph, mvt );
    },


  } );

} );

