// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Dimension = require( 'Dimension' );
  var Font = require( 'Font' );
  var JFrame = require( 'JFrame' );
  var WindowConstants = require( 'WindowConstants' );
  var Property = require( 'Property' );
  var DoubleRange = require( 'DoubleRange' );
  var PhetFont = require( 'PhetFont' );
  var PhetPCanvas = require( 'PhetPCanvas' );
  var LGColors = require( 'LGColors' );
  var Line = require( 'Line' );
  var PointSlopeEquationNode = require( 'PointSlopeEquationNode' );
  var SlopeEquationNode = require( 'SlopeEquationNode' );
  var SlopeInterceptEquationNode = require( 'SlopeInterceptEquationNode' );
  var PNode = require( 'PNode' );

//Class declaration for TestEquationNodes
  return inherit( Object, TestEquationNodes, {

    main: function( args ) {
      var interactiveFont = new PhetFont( this.BOLD, 28 );
      var staticFont = new PhetFont( this.PLAIN, interactiveFont.getSize() );
      var staticColor = this.BLACK;
      var range = new DoubleRange( -10, 10 );
      var x1Range = new Property( range );
      var y1Range = new Property( range );
      var riseRange = new Property( range );
      var runRange = new Property( range );
      var xSpacing = 100;
      var ySpacing = 60;
      var pointSlopeParent = new PNode();
      {
        var line = new Property( Line.createPointSlope( 1, 2, 3, 4, this.INTERACTIVE_LINE ) );
        var equationNode1 = new PointSlopeEquationNode( line, x1Range, y1Range, riseRange, runRange, true, true, true, interactiveFont, staticFont, staticColor );
        var equationNode2 = new PointSlopeEquationNode( line, x1Range, y1Range, riseRange, runRange, false, false, true, interactiveFont, staticFont, staticColor );
        var equationNode3 = new PointSlopeEquationNode( line, x1Range, y1Range, riseRange, runRange, true, true, false, interactiveFont, staticFont, staticColor );
        var equationNode4 = new PointSlopeEquationNode( line, x1Range, y1Range, riseRange, runRange, false, false, false, interactiveFont, staticFont, staticColor );
        var equationNode5 = new SlopeInterceptEquationNode( line, riseRange, runRange, y1Range, false, false, interactiveFont, staticFont, staticColor );
        pointSlopeParent.addChild( equationNode1 );
        pointSlopeParent.addChild( equationNode2 );
        pointSlopeParent.addChild( equationNode3 );
        pointSlopeParent.addChild( equationNode4 );
        pointSlopeParent.addChild( equationNode5 );
        equationNode1.setOffset( 0, 0 );
        equationNode2.setOffset( equationNode1.getXOffset(), equationNode1.getFullBoundsReference().getMaxY() + ySpacing );
        equationNode3.setOffset( equationNode1.getXOffset(), equationNode2.getFullBoundsReference().getMaxY() + ySpacing );
        equationNode4.setOffset( equationNode1.getXOffset(), equationNode3.getFullBoundsReference().getMaxY() + ySpacing );
        equationNode5.setOffset( equationNode1.getXOffset(), equationNode4.getFullBoundsReference().getMaxY() + ySpacing );
      }
      var slopeInterceptParent = new PNode();
      {
        var line = new Property( Line.createSlopeIntercept( 1, 1, 1, this.INTERACTIVE_LINE ) );
        var equationNode1 = new SlopeInterceptEquationNode( line, riseRange, runRange, y1Range, true, true, interactiveFont, staticFont, staticColor );
        var equationNode2 = new SlopeInterceptEquationNode( line, riseRange, runRange, y1Range, true, false, interactiveFont, staticFont, staticColor );
        var equationNode3 = new SlopeInterceptEquationNode( line, riseRange, runRange, y1Range, false, true, interactiveFont, staticFont, staticColor );
        var equationNode4 = new SlopeInterceptEquationNode( line, riseRange, runRange, y1Range, false, false, interactiveFont, staticFont, staticColor );
        slopeInterceptParent.addChild( equationNode1 );
        slopeInterceptParent.addChild( equationNode2 );
        slopeInterceptParent.addChild( equationNode3 );
        slopeInterceptParent.addChild( equationNode4 );
        equationNode1.setOffset( 0, 0 );
        equationNode2.setOffset( equationNode1.getXOffset(), equationNode1.getFullBoundsReference().getMaxY() + ySpacing );
        equationNode3.setOffset( equationNode1.getXOffset(), equationNode2.getFullBoundsReference().getMaxY() + ySpacing );
        equationNode4.setOffset( equationNode1.getXOffset(), equationNode3.getFullBoundsReference().getMaxY() + ySpacing );
      }
      var slopeParent = new PNode();
      {
        var line = new Property( new Line( 1, 3, 2, 4, this.INTERACTIVE_LINE ) );
        var equationNode1 = new SlopeEquationNode( line, x1Range, y1Range, interactiveFont, staticFont, staticColor );
        var equationNode2 = new SlopeEquationNode( line, staticFont, staticColor );
        slopeParent.addChild( equationNode1 );
        slopeParent.addChild( equationNode2 );
        equationNode1.setOffset( 0, 0 );
        equationNode2.setOffset( equationNode1.getXOffset(), equationNode1.getFullBoundsReference().getMaxY() + ySpacing );
      }
      var canvas = new PhetPCanvas();
      canvas.setPreferredSize( new Dimension( 1024, 768 ) );
      canvas.getLayer().addChild( pointSlopeParent );
      canvas.getLayer().addChild( slopeInterceptParent );
      canvas.getLayer().addChild( slopeParent );
      pointSlopeParent.setOffset( 50, 50 );
      slopeInterceptParent.setOffset( pointSlopeParent.getFullBoundsReference().getMaxX() + xSpacing, pointSlopeParent.getYOffset() );
      slopeParent.setOffset( slopeInterceptParent.getXOffset(), slopeInterceptParent.getFullBoundsReference().getMaxY() + 75 );
      var frame = new JFrame( TestEquationNodes.class.getName() );
      frame.setContentPane( canvas );
      frame.pack();
      frame.setDefaultCloseOperation( this.EXIT_ON_CLOSE );
      frame.setVisible( true );
    },


  } );

} );

